from __future__ import annotations

import mknodes as mk


EXPECTED = """!!! info
    This is a test
"""


def test_admonition():
    admonition = mk.MkAdmonition("")
    assert not str(admonition)


def test_markdown():
    admonition = mk.MkAdmonition("This is a test")
    assert str(admonition) == EXPECTED



from __future__ import annotations

import mknodes as mk


EXPECTED = """/// tab | Tab1
    new: True

Some text
///

/// tab | Tab2

Another text
///
"""


def test_tabblock():
    tabs = dict(Tab1="Some text", Tab2="Another text")
    tabblock = mk.MkTabbedBlocks(tabs)
    assert str(tabblock) == EXPECTED



from __future__ import annotations

import types

from typing import Any

from mknodes import treelib
from mknodes.templatenodes import mktreeview
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


class MkModuleOverview(mktreeview.MkTreeView):
    """Node to display a module and its submodules as a tree.

    Submodule names and the first docstring line are shown for each
    submodule.
    """

    ICON = "material/file-tree-outline"
    STATUS = "new"

    def __init__(
        self,
        module: types.ModuleType | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            module: Module to display.
            kwargs: Keyword arguments passed to parent
        """
        super().__init__("", **kwargs)
        self._module = module

    @property
    def tree(self):
        match self._module:
            case types.ModuleType():
                return self._module
            case _:
                return self.ctx.metadata.module

    @tree.setter
    def tree(self, value):
        self._module = value

    @property
    def text(self):
        if not self.tree:
            return ""
        node = treelib.ModuleNode.from_module(
            self.tree,
            predicate=self.predicate,
            exclude=self.exclude_folders,
        )
        return node.get_tree_repr(
            style=self.style,
            max_depth=self.maximum_depth or 0,
        )

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            module=self.tree,
            style=self._style,
            maximum_depth=self.maximum_depth,
            _filter_empty=True,
        )

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkModuleOverview(maximum_depth=2)
        page += mk.MkReprRawRendered(node, header="### From project")

        import git

        node = MkModuleOverview(git, maximum_depth=2)
        page += mk.MkReprRawRendered(node, header="### Explicit")


if __name__ == "__main__":
    import mknodes as mk

    node = MkModuleOverview(mk, header="test", style="ascii")
    print(node.to_markdown())



from __future__ import annotations

import functools
import pathlib

from typing import Any, Literal

from mknodes.basenodes import mktext
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


@functools.cache
def get_markdown() -> str:
    file = pathlib.Path(__file__).parent / "code_of_conduct_2_1.md"
    return file.read_text()


class MkCodeOfConduct(mktext.MkText):
    """Node for a code of conduct section."""

    ICON = "octicons/code-of-conduct-24"

    def __init__(
        self,
        contact_email: str | None = None,
        version: Literal["2.1"] | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            contact_email: Email for contacting. If None, it will be pulled from Project.
            version: Contributor covenant version (currently only "2.1")
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self.version = version
        self.contact_email = contact_email

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            contact_email=self.contact_email,
            version=self.version,
            _filter_empty=True,
        )

    @property
    def text(self) -> str:
        match self.contact_email:
            case str():
                mail = self.contact_email
            case _:
                mail = self.ctx.metadata.author_email or "<MAIL NOT SET>"
        return get_markdown().replace("[INSERT CONTACT METHOD]", mail)

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkCodeOfConduct(contact_email="my@email.com")
        page += mk.MkReprRawRendered(node, header="### Explicit email")
        node = MkCodeOfConduct()
        page += mk.MkReprRawRendered(node, header="### Email from project")


if __name__ == "__main__":
    coc = MkCodeOfConduct("my@email.com")
    print(coc)



from __future__ import annotations

import contextlib
import functools
import inspect
import itertools
import pathlib
import types

from mknodes.data import datatypes
from mknodes.utils import helpers, log


RESPONSE_CODE_OK = 200

logger = log.get_logger(__name__)


def get_stack_info(frame, level: int) -> dict | None:
    for _ in range(level):
        frame = frame.f_back
    if not frame:
        return None
    fn_name = qual if (qual := frame.f_code.co_qualname) != "<module>" else None
    return dict(
        source_filename=frame.f_code.co_filename,
        source_function=fn_name,
        source_line_no=frame.f_lineno,
        # klass=frame.f_locals["self"].__class__.__name__,
    )


@functools.cache
def get_function_body(func: types.MethodType | types.FunctionType | type) -> str:
    """Get body of given function. Strips off the signature.

    Arguments:
        func: Callable to get the body from
    """
    # see https://stackoverflow.com/questions/38050649
    source_lines, _ = get_source_lines(func)
    source_lines = itertools.dropwhile(lambda x: x.strip().startswith("@"), source_lines)
    line = next(source_lines).strip()  # type: ignore
    if not line.startswith(("def ", "class ")):
        return line.rsplit(":")[-1].strip()
    if not line.endswith(":"):
        for line in source_lines:
            line = line.strip()
            if line.endswith(":"):
                break
    return "".join(source_lines)


def get_deprecated_message(obj) -> str | None:
    """Return deprecated message (created by deprecated decorator).

    Arguments:
        obj: Object to check
    """
    return obj.__deprecated__ if hasattr(obj, "__deprecated__") else None


@functools.cache
def get_doc(
    obj,
    *,
    escape: bool = False,
    fallback: str = "",
    from_base_classes: bool = False,
    only_summary: bool = False,
) -> str:
    """Get __doc__ for given object.

    Arguments:
        obj: Object to get docstrings from
        escape: Whether docstrings should get escaped
        fallback: Fallback in case docstrings dont exist
        from_base_classes: Use base class docstrings if docstrings dont exist
        only_summary: Only return first line of docstrings
    """
    if from_base_classes:
        doc = inspect.getdoc(obj)
    else:
        doc = inspect.cleandoc(obj.__doc__) if isinstance(obj.__doc__, str) else None
    if not doc:
        return fallback
    if only_summary:
        doc = doc.split("\n")[0]
    return helpers.escaped(doc) if doc and escape else doc


@functools.cache
def get_source(obj: datatypes.HasCodeType) -> str:
    """Cached wrapper for inspect.getsource.

    Arguments:
        obj: Object to return source for.
    """
    return inspect.getsource(obj)


@functools.cache
def get_source_lines(obj: datatypes.HasCodeType) -> tuple[list[str], int]:
    """Cached wrapper for inspect.getsourcelines.

    Arguments:
        obj: Object to return source lines for.
    """
    return inspect.getsourcelines(obj)


@functools.cache
def get_file(obj: datatypes.HasCodeType) -> pathlib.Path | None:
    """Cached wrapper for inspect.getfile.

    Arguments:
        obj: Object to get file for
    """
    with contextlib.suppress(TypeError):
        return pathlib.Path(inspect.getfile(obj))
    return None



from __future__ import annotations

from abc import ABCMeta
from collections.abc import MutableMapping
import types

import griffe

from griffe.dataclasses import Alias
from griffe.enumerations import Parser
from griffe.loader import GriffeLoader

from mknodes.utils import log


logger = log.get_logger(__name__)


def get_module(module: str | types.ModuleType) -> griffe.Module | Alias:
    """Return info for given module from registry.

    Arguments:
        module: Name of the module
    """
    return registry.get_module(module)


class GriffeRegistry(MutableMapping, metaclass=ABCMeta):
    """Registry for PackageInfos.

    Used for caching all loaded Package information.
    The registry will always only create the Griffe module for the top-level module
    and then use griffe_module[submodule] or griffe_module[klass] to get the
    griffe instances. That should enable the best cache behaviour.

    Examples:
        ``` py
        reg = GriffeRegistry()
        griffe_module = reg.get_module("my_module")
        another_module = reg.get_module("my_module.submodule")
        ```
    """

    def __init__(self):
        self._modules: dict[str, griffe.Module] = {}

    def __getitem__(self, value):
        return self._modules.__getitem__(value)

    def __setitem__(self, index, value):
        self._modules[index] = value

    def __delitem__(self, index):
        del self._modules[index]

    def __repr__(self):
        return f"{type(self).__name__}()"

    def __iter__(self):
        return iter(self._modules.keys())

    def __len__(self):
        return len(self._modules)

    def get_module(
        self,
        module: str | types.ModuleType,
        docstring_style: str = "google",
    ) -> griffe.Module | Alias:
        """Get griffe Module for given module.

        Arguments:
            module: Module to get griffe object for
            docstring_style: Docstring style
        """
        if isinstance(module, types.ModuleType):
            module = module.__name__
        if "." in module:
            module_name, sub_mod_path = module.split(".", 1)
        else:
            module_name, sub_mod_path = module, ""
        if module_name not in self._modules:
            parser = Parser(docstring_style)
            loader = GriffeLoader(docstring_parser=parser)
            self._modules[module_name] = loader.load_module(module_name)
        griffe_mod = self._modules[module_name]
        return griffe_mod[sub_mod_path] if sub_mod_path else griffe_mod

    def get_class(
        self,
        klass: str | type,
        docstring_style: str = "google",
    ) -> griffe.Class | Alias:
        """Get griffe Class for given class.

        Arguments:
            klass: Class to get Griffe object for
            docstring_style: Docstring style
        """
        if isinstance(klass, type):
            mod_name, sub_mod_path = klass.__module__.split(".", 1)
            qual_name = klass.__qualname__
            kls_name = f"{sub_mod_path}.{qual_name}" if sub_mod_path else qual_name
        else:
            mod_name, kls_name = klass.split(".", 1)
        module = self.get_module(mod_name, docstring_style=docstring_style)
        return module[kls_name]


registry = GriffeRegistry()


if __name__ == "__main__":
    reg = GriffeRegistry()
    reg.get_module("mknodes.basenodes")
    info = reg.get_class("mknodes.MkAdmonition")



"""MkNodes routing example.

MkNodes also supports setting up Navs via decorators.
"""

import mknodes as mk


NAV_TEXT = """You can also use decorators to attach MkNavs. These navs then can continue
to build the tree without using decorators (by adding sub-navs).
"""

nav = mk.MkNav("Using decorators")


@nav.route.page("Routed page")
def routed_page(page: mk.MkPage):
    """Builds a MkPage and attaches it to the router MkNav."""
    page += mk.MkAdmonition("I'm a page added via decorators!")


@nav.route.page("Routed", "Deeply", "Nested", "Nested page")
def routed_nested_page(page: mk.MkPage):
    """Builds a nested MkPage and attaches it to the router MkNav."""
    page += mk.MkAdmonition("I'm a nested page added via decorators!")


@nav.route.nav("Routed", "Deeply", "Nested", "Nav")
def routed_nav(nav: mk.MkNav):
    """Builds a nested MkNav and attaches it to the router MkNav."""
    index_page = nav.add_page(is_index=True)
    index_page += mk.MkAdmonition(NAV_TEXT)
    page = nav.add_page("Routed section page")
    page += mk.MkAdmonition("Routed section page content")



from __future__ import annotations

import importlib.util
import io
import logging
import sys


class Logger:
    def __init__(self, logger: logging.Logger):
        self.logger = logger

    def __getattr__(self, val):
        return getattr(self.logger, val)

    def log_dict(self, dct: dict, level=logging.INFO):
        for k, v in dct.items():
            self.logger.log(level, "%s: %s", k, v)


def get_logger(name: str | None = None):
    if importlib.util.find_spec("mkdocs"):
        from mkdocs.plugins import get_plugin_logger

        return get_plugin_logger("mknodes")
    return logging.getLogger(name)


def basic():
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)


log_stream = io.StringIO()
log_handler = logging.StreamHandler(log_stream)
log_handler.setLevel(logging.DEBUG)
fmt = logging.Formatter(fmt="%(asctime)s [%(levelname)s] %(name)s: %(message)s")
log_handler.setFormatter(fmt)
logger = logging.getLogger("mkdocs")
logger.addHandler(log_handler)



from __future__ import annotations

import abc
import collections.abc
import dataclasses

from typing import Any, ClassVar, Literal

from mknodes.utils import helpers, mergehelpers, reprhelpers


@dataclasses.dataclass(frozen=True)
class Package:
    """A python package requirement."""

    package_name: str
    extras: list[str] = dataclasses.field(default_factory=list)


class Extension(dict):
    """A markdown extension resource."""

    def __init__(self, extension_name: str, **kwargs: Any):
        """Constructor.

        Arguments:
            extension_name: Name of the extension
            kwargs: Optional settings for the extension
        """
        super().__init__(**kwargs)
        self.extension_name = extension_name

    def __str__(self):
        return self.extension_name

    def __repr__(self):
        return f"{type(self).__name__}({self.extension_name!r})"

    def __hash__(self):
        return hash(self.extension_name + str(dict(self)))

    def as_mkdocs_dict(self) -> dict[str, dict]:
        return {self.extension_name: dict(self)}


class Plugin(dict):
    """A plugin resource."""

    def __init__(self, plugin_name: str, **kwargs: Any):
        """Constructor.

        Arguments:
            plugin_name: Name of the plugin
            kwargs: Optional settings for the plugin
        """
        super().__init__(**kwargs)
        self.plugin_name = plugin_name

    def __str__(self):
        return self.plugin_name

    def __repr__(self):
        return f"{type(self).__name__}({self.plugin_name!r})"

    def __hash__(self):
        return hash(self.plugin_name + str(dict(self)))

    def as_mkdocs_dict(self) -> dict[str, dict]:
        return {self.plugin_name: dict(self)}


@dataclasses.dataclass(frozen=True)
class CSSFile:
    """CSS file resource."""

    link: str
    color_scheme: Literal["dark", "light"] | None = None
    onload: str | None = None

    def __repr__(self):
        return reprhelpers.dataclass_repr(self)

    def __str__(self):
        return self.link

    def __fspath__(self):
        return self.link

    def is_local(self) -> bool:
        """Whether the css file is a local resource."""
        return not helpers.is_url(self.link)

    def to_html(self) -> str:
        """Return html "link" element which can be put into an HTML page."""
        if self.color_scheme == "light":
            media = ' media="(prefers-color-scheme:light)"'
        elif self.color_scheme == "dark":
            media = ' media="(prefers-color-scheme:dark)"'
        else:
            media = ""
        onload = f" onload={self.onload!r}" if self.onload else ""
        # return '<link rel="stylesheet" href="{{ base_url }}/{path}" />'
        return f'<link rel="stylesheet" href={self.link!r}{media}{onload}/>'


@dataclasses.dataclass(frozen=True)
class JSFile:
    """A JavaScript link resource."""

    link: str
    defer: bool = False
    async_: bool = False
    crossorigin: Literal["anonymous", "use-credentials"] | None = None
    typ: str = ""
    is_library: bool = False

    def __repr__(self):
        return reprhelpers.dataclass_repr(self)

    def __str__(self):
        return self.link

    def __fspath__(self):
        return str(self)

    def is_local(self) -> bool:
        """Whether the js file is a local resource."""
        return not helpers.is_url(self.link)

    def to_html(self) -> str:
        """Return html "script" element which can be put into an HTML page."""
        html = f'<script src="{self.link}"'
        if self.typ:
            html += f' type="{self.typ}"'
        if self.defer:
            html += " defer"
        if self.async_:
            html += " async"
        if self.crossorigin:
            html += f' crossorigin="{self.crossorigin}"'
        html += "></script>"
        return html


class TextResource:
    EXTENSION: str
    filename: str | None
    content: str

    @property
    def resolved_filename(self) -> str:
        """Return a filename consisting of a user-set prefix + a hash value."""
        hashed = helpers.get_hash(self.content)
        return (
            f"{self.filename.removesuffix(self.EXTENSION)}_{hashed}{self.EXTENSION}"
            if self.filename
            else f"{hashed}{self.EXTENSION}"
        )

    def __hash__(self):
        return hash(self.content)

    def __repr__(self):
        return reprhelpers.get_repr(self, filename=self.resolved_filename)


@dataclasses.dataclass(frozen=True, repr=False)
class CSSText(TextResource):
    """Class representing non-file-bound CSS."""

    EXTENSION: ClassVar = ".css"
    content: str
    filename: str | None

    def to_html(self) -> str:
        # content = self.content.replace("\n", "")
        return f"<style>\n{self.content}\n</style>"


@dataclasses.dataclass(frozen=True, repr=False)
class JSText(TextResource):
    """Class representing non-file-bound JavaScript code."""

    EXTENSION: ClassVar = ".js"
    content: str
    filename: str | None
    defer: bool = False
    async_: bool = False
    crossorigin: Literal["anonymous", "use-credentials"] | None = None
    typ: str = ""
    is_library: bool = False

    def get_jsfile(self) -> JSFile:
        return JSFile(
            link=self.resolved_filename,
            async_=self.async_,
            defer=self.defer,
            crossorigin=self.crossorigin,
            typ=self.typ,
            is_library=self.is_library,
        )

    def get_asset(self) -> Asset:
        return Asset(filename=self.resolved_filename, content=self.content)


class Asset:
    """An asset resource."""

    def __init__(self, filename: str, content: str, target: str | None = None):
        """Constructor.

        Arguments:
            filename: Filename for the asset
            content: File content
            target: target directory for the asset file
        """
        self.filename = filename
        self.content = content
        self.target_dir = target

    def __hash__(self):
        return hash(self.content)

    def __repr__(self):
        return f"{type(self).__name__}({self.filename!r})"


@dataclasses.dataclass
class Resources(collections.abc.Mapping, metaclass=abc.ABCMeta):
    """A resource bundle containing different assets.

    Most of the time this class is used for bundling required resources
    for a specific node.
    """

    css: list[CSSType] = dataclasses.field(default_factory=list)
    """A filepath->filecontent dictionary containing the required CSS."""
    markdown_extensions: dict[str, dict] = dataclasses.field(default_factory=dict)
    """A extension_name->settings dictionary containing the required md extensions."""
    plugins: list[Plugin] = dataclasses.field(default_factory=list)
    """A set of required plugins. (Only for info purposes)"""
    js: list[JSType] = dataclasses.field(default_factory=list)
    """A list of JS file paths."""
    assets: list[Asset] = dataclasses.field(default_factory=list)
    """A list of additional assets required (static files which go into assets dir)."""
    packages: list[Package] = dataclasses.field(default_factory=list)
    """A list of required packages."""

    def __getitem__(self, value):
        return getattr(self, value)

    def __setitem__(self, index, value):
        setattr(self, index, value)

    def __contains__(self, other):
        if any(i == other for i in self.markdown_extensions):
            return True
        if any(i.plugin_name == other for i in self.plugins):
            return True
        return super().__contains__(other)

    def __len__(self):
        return len(dataclasses.fields(self))

    def __iter__(self):
        return iter(i.name for i in dataclasses.fields(self))

    def __ior__(self, other):
        return self.merge(other)

    def __or__(self, other):
        return self.merge(other)

    def remove(self, *resources):
        for resource in resources:
            if resource in self.css:
                self.css.remove(resource)
            if resource in self.plugins:
                self.plugins.remove(resource)
            if resource in self.js:
                self.js.remove(resource)
            if resource in self.assets:
                self.assets.remove(resource)
            if resource in self.packages:
                self.packages.remove(resource)
            if resource in self.markdown_extensions:
                self.markdown_extensions.pop(resource)

    @property
    def js_files(self) -> list[JSText]:
        """All JavaScript files of this resource bundle."""
        return [i for i in self.js if isinstance(i, JSText)]

    @property
    def js_links(self) -> list[JSFile]:
        """All JavaScript links of this resource bundle."""
        return [i for i in self.js if isinstance(i, JSFile)]

    @property
    def js_libs(self) -> list[JSFile | JSText]:
        """All JavaScript links of this resource bundle."""
        return [i for i in self.js if i.is_library]

    def merge(self, other: collections.abc.Mapping, additive: bool = False):
        """Merge resources with another resources instance or dict.

        Adds resources from other to this instance.

        Arguments:
            other: The resources to merge into this one.
            additive: Merge strategy. Either additive or replace.
        """
        if other_exts := other["markdown_extensions"]:
            exts = [self.markdown_extensions, other_exts]
            merged = mergehelpers.merge_extensions(exts)
            self.markdown_extensions = mergehelpers.merge_dicts(*merged)
        self.css = helpers.reduce_list(self.css + other["css"])
        self.plugins = helpers.reduce_list(self.plugins + other["plugins"])
        self.js = helpers.reduce_list(self.js + other["js"])
        self.assets = helpers.reduce_list(self.assets + other["assets"])
        self.packages = helpers.reduce_list(self.packages + other["packages"])
        return self


CSSType = CSSFile | CSSText
JSType = JSFile | JSText


if __name__ == "__main__":
    link = JSText("jkfdjl", "kfjdsdkljf", async_=True)
    print(link)



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mknode
from mknodes.utils import icons, log, reprhelpers


logger = log.get_logger(__name__)


class MkIcon(mknode.MkNode):
    """Pyconify-based Icon."""

    ICON = "material/image"
    ATTR_LIST_SEPARATOR = "\n"
    STATUS = "new"

    def __init__(
        self,
        icon_name: str,
        *,
        color: str | None = None,
        height: str | int | None = None,
        width: str | int | None = None,
        flip: icons.Flip | None = None,
        rotate: icons.Rotation | None = None,
        box: bool | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            icon_name: Icon name
            color: Icon color. Replaces currentColor with specific color, resulting in
                   icon with hardcoded palette.
            height: Icon height. If only one dimension is specified, such as height, other
                    dimension will be automatically set to match it.
            width: Icon width. If only one dimension is specified, such as height, other
                   dimension will be automatically set to match it.
            flip: Flip icon.
            rotate: Rotate icon. If an int is provided, it is assumed to be in degrees.
            box: Adds an empty rectangle to SVG that matches the icon's viewBox.
                 It is needed when importing SVG to various UI design tools that ignore
                 viewBox. Those tools, such as Sketch, create layer groups that
                 automatically resize to fit content. Icons usually have empty pixels
                 around icon, so such software crops those empty pixels and icon's
                 group ends up being smaller than actual icon, making it harder to
                 align it in design.
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self.icon_name = icon_name
        self.color = color
        self.height = height
        self.width = width
        self.flip = flip
        self.rotate = rotate
        self.box = box

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            icon_name=self.icon_name,
            color=self.color,
            height=self.height,
            width=self.width,
            flip=self.flip,
            rotate=self.rotate,
            box=self.box,
            _filter_empty=True,
            _filter_false=True,
        )

    def _to_markdown(self) -> str:
        return icons.get_icon_svg(
            self.icon_name,
            color=self.color,
            height=self.height,
            width=self.width,
            flip=self.flip,
            rotate=self.rotate,
            box=self.box,
        )

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkIcon(icon_name="mdi:file")
        page += mk.MkReprRawRendered(node, header="### With caption")

        node = MkIcon(icon_name="mdi:file", color="red")
        page += mk.MkReprRawRendered(node, header="### Colored")

        node = MkIcon(icon_name="mdi:file", rotate=90)
        page += mk.MkReprRawRendered(node, header="### Rotated")

        node = MkIcon(icon_name="noto:wrench", height=100, width=100)
        page += mk.MkReprRawRendered(node, header="### Sized")


if __name__ == "__main__":
    img = MkIcon("mdi:file", rotate="90")
    print(img)



from __future__ import annotations

import re

from typing import Literal, Self

from mknodes.data import datatypes
from mknodes.utils import yamlhelpers


HEADER = "---\n{options}---\n"

HEADER_RE = re.compile(r"\A-{3}\n([\S\s]*)^-{3}(\n|$)", re.MULTILINE)


SectionStr = Literal["navigation", "toc", "path", "tags", "footer", "feedback"]


class Metadata(dict):
    """The Metadata class is a subclassed Python dictionary.

    It is enhanced with properties for common metadata fields in order
    to get proper type hints. Since this object is a dict subclass, you can of
    course also add any non-documented stuff to the Metadata.
    """

    def __init__(self, *args, **kwargs):
        search_dict = {}
        if "search_boost" in kwargs and kwargs["search_boost"] is not None:
            search_dict["boost"] = kwargs.pop("search_boost")
        if "exclude_from_search" in kwargs and kwargs["exclude_from_search"] is not None:
            search_dict["exclude"] = kwargs.pop("exclude_from_search")
        if search_dict:
            kwargs["search"] = search_dict
        super().__init__(*args, **kwargs)
        if self.icon and "/" not in self.icon and ":" not in self.icon:
            self.icon = f"material/{self.icon}"
        if isinstance(self.hide, str):
            self.hide = [i.strip() for i in self.hide.split(",")]
        if self.hide is not None:
            self.hide = [i if i != "nav" else "navigation" for i in self.hide or []]

    @property
    def hide(self) -> list[SectionStr] | None:
        """A list of items which should be hidden from the page.

        MkDocs-Material supports `"navigation"`, `"toc"`, `"path"`, `"tags"`.
        """
        return self.get("hide")

    @hide.setter
    def hide(self, val: list[SectionStr] | str | None):
        self["hide"] = val

    @property
    def search_boost(self) -> float | None:
        """A multiplier to modify search relevance."""
        return self.get("search_boost")

    @search_boost.setter
    def search_boost(self, val: float | None):
        self["search_boost"] = val

    @property
    def exclude_from_search(self) -> bool | None:
        """Exclude this page from search."""
        return self.get("exclude_from_search")

    @exclude_from_search.setter
    def exclude_from_search(self, val: bool | None):
        self["exclude_from_search"] = val

    @property
    def revision_date(self) -> bool | None:
        """Date when page was last revised."""
        return self.get("git_revision_date_localized")

    @revision_date.setter
    def revision_date(self, val: bool | None):
        self["git_revision_date_localized"] = val

    @property
    def creation_date(self) -> bool | None:
        """Page creation date."""
        return self.get("git_creation_date_localized")

    @creation_date.setter
    def creation_date(self, val: bool | None):
        self["git_creation_date_localized"] = val

    @property
    def icon(self) -> str | None:
        """An icon for the page (Example: `"material/wrench"`)."""
        return self.get("icon")

    @icon.setter
    def icon(self, val: str | None):
        # if val and "/" not in val:
        #     val = f"material/{val}"
        self["icon"] = val

    @property
    def status(self) -> datatypes.PageStatusStr | str | None:
        """The status of the page.

        MkDocs-Material supports `"new"`, `"deprecated"` and `"encrypted"` by default
        by showing an icon in the site nav. Additional icons can be added during
        the build process via
        [MaterialTheme.add_status_icon][mknodes.MaterialTheme.add_status_icon].
        """
        return self.get("status")

    @status.setter
    def status(self, val: datatypes.PageStatusStr | None):
        self["status"] = val

    @property
    def title(self) -> str | None:
        """Title for the page."""
        return self.get("title")

    @title.setter
    def title(self, val: str | None):
        self["title"] = val

    @property
    def subtitle(self) -> str | None:
        """Subtitle for the page."""
        return self.get("subtitle")

    @subtitle.setter
    def subtitle(self, val: str | None):
        self["subtitle"] = val

    @property
    def description(self) -> str | None:
        """Description text for the page."""
        return self.get("description")

    @description.setter
    def description(self, val: str | None):
        self["description"] = val

    @property
    def inclusion_level(self) -> str | None:
        """Page inclusion level."""
        return self.get("inclusion_level")

    @inclusion_level.setter
    def inclusion_level(self, val: str | None):
        self["inclusion_level"] = val

    @property
    def template(self) -> str | None:
        """Filename of the template the page should use.

        This is only a reference, you still need to add the template to the theme.
        """
        return self.get("template")

    @template.setter
    def template(self, val: str | None):
        self["template"] = val

    @property
    def tags(self) -> list[str] | None:
        """A list of tags associated with the page."""
        return self.get("tags")

    @tags.setter
    def tags(self, val: list[str] | None):
        self["tags"] = val

    @property
    def search(self) -> dict | None:
        """A dictionary containing search-related settings (`"boost"` / `"exclude"`)."""
        return self.get("search")

    @search.setter
    def search(self, val: dict | None):
        self["search"] = val

    @classmethod
    def parse(cls, text: str) -> tuple[Self, str]:
        """Parse given text for metadata and return a (Metadata, Rest-from-text) tuple."""
        dct = {}
        if match := HEADER_RE.match(text):
            content = match[1]
            dct = yamlhelpers.load_yaml(content)
            text = text[match.span()[1] :]
        return cls(**dct), text

    def __str__(self):
        dct = {k: v for k, v in self.items() if v is not None}
        return yamlhelpers.dump_yaml(dct) if dct else ""

    def as_page_header(self) -> str:
        """Format metadata in Page header format (with --- separators)."""
        text = str(self)
        return HEADER.format(options=text) if text else ""


if __name__ == "__main__":
    metadata = Metadata(hide="toc", search_boost=2)
    print(metadata)



from __future__ import annotations

from functools import cached_property
import tempfile

from typing import Self
from urllib import parse

import git

from mknodes.info import contexts
from mknodes.utils import log


logger = log.get_logger(__name__)


class GitRepository(git.Repo):
    """Aggregates information about a git repo."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # to keep a reference to a TempDirectory instance
        self.temp_directory: tempfile.TemporaryDirectory | None = None

    def __len__(self):
        return len(list(self.iter_commits("HEAD")))

    @cached_property
    def main_branch(self) -> str:
        """The default branch of the repository."""
        has_master_branch = any(branch.name == "master" for branch in self.heads)
        return "master" if has_master_branch else "main"

    @classmethod
    def clone_from(cls, *args, **kwargs) -> Self:
        """Clone a repository. Overridden for typing."""
        return super().clone_from(*args, **kwargs)  # type: ignore[return-value]

    @cached_property
    def repo_name(self) -> str:
        """Name (aka the last part of the url) of the Git repository."""
        return self.remotes.origin.url.split(".git")[0].split("/")[-1]

    @cached_property
    def repo_url(self) -> str:
        """Url of the remote repository (without .git)."""
        return self.remotes.origin.url.split(".git")[0] + "/"

    @cached_property
    def commit_to_tag(self) -> dict[git.Commit, str]:  # type: ignore[name-defined]
        """Dictionary mapping git.Commits to tag strings."""
        return {self.commit(i.commit): i.name for i in self.tags}

    def get_version_for_commit(
        self,
        commit: git.Commit | str,  # type: ignore[name-defined]
    ) -> str | None:
        """Iterate commit parents to find the associated version of the commit.

        Arguments:
            commit: Commit to get a version for.
        """
        if isinstance(commit, str):
            commit = self.commit(commit)
        return next(
            (
                self.commit_to_tag[c]
                for c in [commit, *commit.parents]
                if c in self.commit_to_tag
            ),
            None,
        )

    def get_last_commits(
        self,
        num: int,
        branch: str | None = None,
    ) -> list[git.Commit]:  # type: ignore[name-defined]
        """Return last x commits.

        Arguments:
            num: Amount of commits to fetch.
            branch: Branch to get commits from. Defaults to main / master.
        """
        rev = branch or self.main_branch
        try:
            return list(self.iter_commits(rev, max_count=num))
        except git.exc.GitCommandError:  # type: ignore[name-defined]
            logger.warning("Could not fetch commits for %r", rev)
            return []

    @cached_property
    def code_repository(self) -> str:
        """Get the remote code repository name (like "GitHub")."""
        repo_host = parse.urlsplit(self.remotes.origin.url).netloc.lower()
        match repo_host:
            case "github.com":
                return "GitHub"
            case "bitbucket.org":
                return "Bitbucket"
            case "gitlab.com":
                return "GitLab"
            case _:
                return repo_host.split(".")[0].title()

    @property
    def edit_uri(self) -> str | None:
        """The URL part needed to get to the edit page of the code hoster."""
        repo_host = parse.urlsplit(self.remotes.origin.url).netloc.lower()
        match repo_host:
            case "github.com" | "gitlab.com":
                return f"edit/{self.main_branch}/"
            case "bitbucket.org":
                return "src/default/"
            case _:
                return None

    @cached_property
    def context(self) -> contexts.GitContext:
        """Return Git context."""
        return contexts.GitContext(
            main_branch=self.main_branch,
            repo_hoster=self.code_repository,
            last_commits=self.get_last_commits(100),
            repo_name=self.repo_name,
            edit_uri=self.edit_uri,
        )


if __name__ == "__main__":
    repo = GitRepository(".")
    # for commit in repo.get_last_commits(100):
    v = repo.get_version_for_commit("949f6df9cdf49d175bfe2c57d8f51d3882c7fc01")
    print(v)



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mkdefinitionlist
from mknodes.utils import helpers, log, reprhelpers, superdict


logger = log.get_logger(__name__)


class MkConfigSetting(mkdefinitionlist.MkDefinition):
    """Node for describing a config setting."""

    ICON = "material/library"

    def __init__(
        self,
        title: str,
        description: str,
        setting: dict[str, Any] | str | None = None,
        default: str | int | None = None,
        version_added: str | None = None,
        optional: bool | None = None,
        mode: superdict.MarkupTypeStr | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            title: Setting title
            description: Setting description
            setting: (Nested) json-like object representing the setting
            default: Default setting value
            version_added: Version added
            optional: Whether setting is optional. (None hides the label)
            mode: Markup language of settings file
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(content=None, title=title, **kwargs)
        self.setting = setting
        self.default = default
        self.version_added = version_added
        self.description = description
        self.optional = optional
        self.mode = mode

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            title=self.title,
            description=self.description,
            default=self.default,
            version_added=self.version_added,
            optional=self.optional,
            mode=self.mode,
            _filter_empty=True,
        )

    @property
    def title(self) -> str:
        return helpers.styled(self._title, bold=True, code=True)

    @property
    def items(self):
        import mknodes as mk

        text = f"Default: `{self.default!r}`\n\n"
        if self.version_added:
            text += f"Version added: `{self.version_added}`\n\n"
        if self.optional is not None:
            required = "no" if self.optional else "yes"
            text += f"Required: `{required}`\n\n"
        text += f"{self.description}\n"
        items: list[mk.MkNode] = [mk.MkText(text, parent=self)]
        if isinstance(self.setting, dict):
            code = superdict.SuperDict(self.setting).serialize(mode=self.mode)
        else:
            code = self.setting
        if self.setting:
            code_node = mk.MkCode(code, parent=self, language=self.mode or "yaml")
            items.append(code_node)
        return items

    @items.setter
    def items(self, value):
        pass

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        setting = {"plugins": {"mknodes": {"path": "path.to.module"}}}
        desc = "Some **markdown** to describe the setting"
        node = MkConfigSetting(
            "path",
            setting=setting,
            description=desc,
            default="path.to.module",
        )
        page += mk.MkReprRawRendered(node, header="### YAML")
        node = MkConfigSetting(
            "path",
            mode="json",
            setting=setting,
            description=desc,
            default="path.to.module",
        )
        page += mk.MkReprRawRendered(node, header="### JSON")
        node = MkConfigSetting(
            "path",
            mode="toml",
            version_added="2.1.0",
            setting=setting,
            description=desc,
            default="path.to.module",
        )
        page += mk.MkReprRawRendered(node, header="### TOML")


if __name__ == "__main__":
    setting = {"path": {"to": "setting"}}
    desc = "Some text to describe the setting"
    node = MkConfigSetting("YAML", description=desc, setting=setting)
    print(node)



from __future__ import annotations

import os
import pathlib

from urllib import parse

import requests
import requests_cache

from mknodes.utils import log


logger = log.get_logger(__name__)


requests_cache.install_cache(
    cache_control=True,
    backend="filesystem",  # default was "memory", not sure what is more suiting.
    use_temp=True,
    urls_expire_after={
        # "*.github.com": 1000,
        "*": 1000,
    },
)


def download(url: str, headers: dict[str, str] | None = None) -> bytes:
    """Downloads a file from given URL returns its content.

    If the URL is a local path, it is simply read and returned instead.

    Arguments:
        url: URL or local path of the file to use.
        headers: Headers to use for the request.
    """
    logger.debug("Getting file for '%s'", url)
    if parse.urlsplit(url).scheme not in ("http", "https"):
        with pathlib.Path(url).open("rb") as f:
            return f.read()
    if token := os.getenv("GH_TOKEN"):
        gh_header = dict(Authorization=f"token {token}")
    else:
        gh_header = {}
    req = requests.get(url, headers=gh_header | (headers or {}))
    logger.debug("Downloaded %s", url)
    return req.content



from __future__ import annotations

import contextlib
import pkgutil

from typing import Any, Literal
from urllib import parse

from mknodes.basenodes import mkcontainer, mknode
from mknodes.data import datatypes
from mknodes.info import packageregistry
from mknodes.templatenodes import mkbadge
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


class MkMetadataBadges(mkcontainer.MkContainer):
    """Container node for a list of metadata badges.

    Badges are generated locally using "anybadge" package
    """

    ICON = "simple/shieldsdotio"
    STATUS = "new"

    def __init__(
        self,
        typ: datatypes.MetadataTypeStr,
        *,
        package: str | None = None,
        font_size: Literal[10, 11, 12] | None = None,
        font_name: str | None = None,
        num_padding_chars: int | None = None,
        badge_color: str | None = None,
        text_color: str | None = None,
        use_gitlab_style: bool = False,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            typ: Metadata badges to show
            package: Package to show badges for. If None, it will get pulled from project
            font_size: Size of font to use
            font_name: Name of font to use
            num_padding_chars: Number of chars to use for padding
            badge_color: Badge color. If none is set, it will be pulled from project.
            text_color: Badge color
            use_gitlab_style: Use Gitlab-scope style
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self.block_separator = "\n"
        self._package = package
        self._typ = typ
        self.font_size = font_size
        self.font_name = font_name
        self.num_padding_chars = num_padding_chars
        self.badge_color = badge_color
        self.text_color = text_color
        self.use_gitlab_style = use_gitlab_style

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            typ=self._typ,
            package=self._package,
            font_size=self.font_size,
            font_name=self.font_name,
            badge_color=self.badge_color,
            text_color=self.text_color,
            num_padding_chars=self.num_padding_chars,
            use_gitlab_style=self.use_gitlab_style,
            _filter_empty=True,
            _filter_false=True,
        )

    @property
    def badge_content(self) -> list[tuple]:
        items: list[tuple] = []
        ctx = (
            packageregistry.get_info(self._package)
            if self._package
            else self.ctx.metadata
        )
        match self._typ:
            case "classifiers":
                dct = ctx.classifier_map
                for category, labels in dct.items():
                    items.extend([(i, category, None) for i in labels])
            case "keywords":
                items.extend((keyword, "", None) for keyword in ctx.keywords)
            case "keywords_combined":
                items.append(("Keywords", " | ".join(ctx.keywords), None))
            case "required_python":
                string = ctx.required_python_version
                items.append(("Python", string, "https://www.python.org"))
            case "websites":
                urls = [
                    (name, parse.urlparse(url).netloc, url)
                    for name, url in ctx.urls.items()
                ]
                items.extend(urls)
            case "dependencies":
                info = ctx.required_packages
                items.extend(
                    (package.name, package.version, package.homepage) for package in info
                )
            case "installed_packages":
                pkgs = []
                for mod in pkgutil.iter_modules():
                    if not mod.ispkg:
                        continue
                    with contextlib.suppress(Exception):
                        dist = packageregistry.get_info(mod.name)
                        pkgs.append(dist)
                items.extend(
                    (package.name, package.version, package.homepage) for package in pkgs
                )
            case str():
                raise ValueError(self._typ)
            case _ if self._typ in datatypes.CLASSIFIERS:
                labels = ctx.classifier_map.get(self._typ, [])
                items.extend([(i, self._typ, None) for i in labels])
        return items

    @property
    def items(self) -> list[mknode.MkNode]:
        return [
            mkbadge.MkBadge(
                label=label,
                value=value,
                link=link,
                font_size=self.font_size,
                font_name=self.font_name,
                badge_color=self.badge_color,
                text_color=self.text_color,
                num_padding_chars=self.num_padding_chars,
                use_gitlab_style=self.use_gitlab_style,
                title=label,
                parent=self,
            )
            for label, value, link in self.badge_content
        ]

    @items.setter
    def items(self, value):
        pass

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkMetadataBadges(typ="classifiers")
        page += mk.MkReprRawRendered(node, header="### Classifiers")
        node = MkMetadataBadges(typ="keywords")
        page += mk.MkReprRawRendered(node, header="### Keywords")
        node = MkMetadataBadges(typ="keywords_combined")
        page += mk.MkReprRawRendered(node, header="### Keywords")
        node = MkMetadataBadges(typ="websites")
        page += mk.MkReprRawRendered(node, header="### Websites")
        node = MkMetadataBadges(typ="dependencies")
        page += mk.MkReprRawRendered(node, header="### Dependencies")
        node = MkMetadataBadges(typ="dependencies", package="mkdocs")
        page += mk.MkReprRawRendered(node, header="### For other package")
        # node = MkMetadataBadges(typ="installed_packages")
        # page += mk.MkReprRawRendered(node, header="### Installed packages")
        node = MkMetadataBadges(typ="classifiers", use_gitlab_style=True)
        page += mk.MkReprRawRendered(node, header="### Gitlab style")
        node = MkMetadataBadges(typ="required_python", badge_color="red")
        page += mk.MkReprRawRendered(node, header="### Colored")


if __name__ == "__main__":
    node = MkMetadataBadges.with_default_context("websites")
    print(node)



from __future__ import annotations

from collections.abc import Sequence
import inspect
import pathlib
import types

from typing import TYPE_CHECKING, Any

from mknodes.basenodes import mknode
from mknodes.data.datatypes import PageStatusStr
from mknodes.navs import navigation, navparser, navrouter
from mknodes.pages import metadata, mkpage, pagetemplate
from mknodes.utils import inspecthelpers, log, reprhelpers, resources


if TYPE_CHECKING:
    import mknodes as mk


logger = log.get_logger(__name__)


class MkNav(mknode.MkNode):
    """Nav section, representing a nestable menu.

    A nav is named (exception is the root nav, which has section name = None),
    has an associated virtual file (in general a SUMMARY.md),
    an optional index page and can contain other navs as well as pages and links.
    """

    ICON = "material/navigation-outline"
    REQUIRED_PLUGINS = [
        resources.Plugin("literate_nav"),
        resources.Plugin("section_index"),
    ]

    def __init__(
        self,
        section: str | None = None,
        *,
        filename: str = "SUMMARY.md",
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            section: Section name for the Nav
            filename: FileName for the resulting nav
            kwargs: Keyword arguments passed to parent
        """
        self.section = section
        self.filename = filename
        self.nav = navigation.Navigation()
        """Navigation object containing all child items."""
        self.route = navrouter.NavRouter(self)
        """Router used for decorator routing."""
        self.parse = navparser.NavParser(self)
        """Parser object used to build Navs from different data / directory structures."""
        self.metadata = metadata.Metadata()
        """Page Metadata."""
        self.page_template = pagetemplate.PageTemplate(parent=self, extends="main.html")
        super().__init__(**kwargs)
        if frame := inspect.currentframe():
            self.metadata["created"] = inspecthelpers.get_stack_info(frame, level=1)

    def __repr__(self):
        section = self.section or "<root>"
        return reprhelpers.get_repr(self, section=section, filename=self.filename)

    # The child items are managed by the Navigation object. We forward relevant calls
    # to the Navigation instance.

    def __setitem__(self, index: tuple | str, node: navigation.NavSubType):
        self.nav[index] = node

    def __getitem__(self, index: tuple | str) -> navigation.NavSubType:
        return self.nav[index]

    def __delitem__(self, index: tuple | str):
        del self.nav[index]

    # def __len__(self):
    #     return len(self.nav.all_items)

    def __iter__(self):
        yield from self.nav.all_items

    @property
    def index_page(self) -> mk.MkPage | None:
        """Get the current index page if set."""
        return self.nav.index_page

    @index_page.setter
    def index_page(self, value):
        value.parent = self
        self.nav.index_page = value

    @property
    def index_title(self):
        return self.nav.index_title

    @index_title.setter
    def index_title(self, value):
        self.nav.index_title = value

    @property
    def children(self):
        return self.nav.all_items

    @children.setter
    def children(self, items):
        self.nav = navigation.Navigation(items)

    def __add__(self, other: navigation.NavSubType):
        """Use this to to register MkNodes."""
        other.parent = self
        self.nav.register(other)
        return self

    @property
    def resolved_file_path(self) -> str:
        """Returns the resulting section/subsection/../filename.xyz path."""
        path = "/".join(self.resolved_parts) + "/" + self.filename
        return path.lstrip("/")

    def add_nav(self, section: str) -> MkNav:
        """Create a Sub-Nav, register it to given Nav and return it.

        Arguments:
            section: Name of the new nav.
        """
        navi = MkNav(section=section, parent=self)
        self.nav.register(navi)
        return navi

    def to_markdown(self) -> str:
        return self.nav.to_literate_nav()

    def add_page(
        self,
        title: str | None = None,
        *,
        is_index: bool = False,
        is_homepage: bool = False,
        path: str | None = None,
        hide: list[str] | str | None = None,
        search_boost: float | None = None,
        exclude_from_search: bool | None = None,
        icon: str | None = None,
        status: PageStatusStr | None = None,
        subtitle: str | None = None,
        description: str | None = None,
        template: str | None = None,
        tags: list[str] | None = None,
    ) -> mk.MkPage:
        """Add a page to the Nav.

        Arguments:
            title: Page title
            is_index: Whether the page should become the index page.
            is_homepage: Whether the page should become the homepage.
            path: optional path override
            hide: Hide parts of the page ("toc", "nav", "path")
            search_boost: multiplier for search ranking
            exclude_from_search: Exclude page from search index
            icon: optional page icon
            status: Page status
            subtitle: Page subtitle
            description: Page description
            template: Page template
            tags: tags to show above the main headline and within the search preview
        """
        page = mkpage.MkPage(
            title=title,
            is_index=is_index,
            is_homepage=is_homepage,
            path=path,
            parent=self,
            hide=hide,
            search_boost=search_boost,
            exclude_from_search=exclude_from_search,
            icon=icon,
            status=status,
            subtitle=subtitle,
            description=description,
            template=template,
            tags=tags,
        )
        if is_index:
            self.index_page = page
            self.index_title = title or self.section or "Home"
        else:
            self.nav.register(page)
        return page

    def add_doc(
        self,
        module: types.ModuleType | Sequence[str] | str | None = None,
        *,
        filter_by___all__: bool = False,
        recursive: bool = False,
        section_name: str | None = None,
        class_page: type[mk.MkClassPage] | str | None = None,
        module_page: type[mk.MkModulePage] | str | None = None,
        flatten_nav: bool = False,
    ) -> mk.MkDoc:
        """Add a module documentation to the Nav.

        Arguments:
            module: The module to create a documentation section for.
            filter_by___all__: Whether the documentation
            recursive: Whether to search modules recursively
            section_name: Override the name for the menu (default: module name)
            class_page: Override for the default ClassPage
            module_page: Override for the default ModulePage
            flatten_nav: Whether classes should be put into top-level of the nav
        """
        import mknodes as mk

        nav = mk.MkDoc(
            module=module,
            filter_by___all__=filter_by___all__,
            parent=self,
            section_name=section_name,
            recursive=recursive,
            class_page=class_page,
            module_page=module_page,
            flatten_nav=flatten_nav,
        )
        self.nav.register(nav)
        return nav


if __name__ == "__main__":
    docs = MkNav()
    nav_tree_path = pathlib.Path(__file__).parent.parent.parent / "tests/data/nav_tree/"
    nav_file = nav_tree_path / "SUMMARY.md"
    nav = MkNav()
    nav.parse.file(nav_file)
    print(nav)



from __future__ import annotations

import dataclasses

from operator import attrgetter
import os
import reprlib
from typing import Any

from mknodes.data import datatypes
from mknodes.utils import log


logger = log.get_logger(__name__)


class LengthLimitRepr(reprlib.Repr):
    def repr_type(self, obj, level):
        return obj.__name__

    def repr_module(self, obj, level):
        return obj.__name__

    def repr_function(self, obj, level):
        return obj.__name__


limit_repr = LengthLimitRepr()
limit_repr.maxlist = 10
limit_repr.maxstring = 100
limit_repr.maxlevel = 8
limit_repr.maxtuple = 10
limit_repr.maxarray = 10
limit_repr.maxdict = 10
limit_repr.maxset = 10
limit_repr.maxfrozenset = 10
limit_repr.maxdeque = 10
limit_repr.maxlong = 60
limit_repr.maxother = 60


def get_repr(
    _obj: Any,
    *args: Any,
    _shorten: bool = True,
    _filter_empty: bool = False,
    _filter_false: bool = False,
    **kwargs: Any,
) -> str:
    """Get a suitable __repr__ string for an object.

    Args:
        _obj: The object to get a repr for.
        *args: Arguments for the repr
        _shorten: Whether to shorten the repr.
        _filter_empty: Filter kwargs  with None, empty str / empty dicts as value
        _filter_false: Filter False values
        **kwargs: Keyword arguments for the repr
    """
    my_repr = limit_repr.repr if _shorten else repr
    classname = type(_obj).__name__
    parts = [my_repr(val) for val in args]
    kw_parts = []
    for k, v in kwargs.items():
        if _filter_empty and (v is None or v == "" or v == {}):
            continue
        if _filter_false and v is False:
            continue

        import mknodes as mk

        match v:
            case (mk.MkNode(), *_) if len(v) > 1:
                name = "[...]"
            case os.PathLike():
                name = repr(os.fspath(v))
            case _:
                name = my_repr(v)
        kw_parts.append(f"{k}={name}")
    sig = ", ".join(parts + kw_parts)
    return f"{classname}({sig})"


def dataclass_repr(instance: datatypes.DataclassInstance) -> str:
    """Return repr for dataclass, filtered by non-default values.

    Arguments:
        instance: dataclass instance
    """
    nodef_f_vals = (
        (f.name, attrgetter(f.name)(instance))
        for f in dataclasses.fields(instance)
        if attrgetter(f.name)(instance) != f.default
    )

    nodef_f_repr = ", ".join(f"{name}={value!r}" for name, value in nodef_f_vals)
    return f"{instance.__class__.__name__}({nodef_f_repr})"


def to_str_if_textnode(node) -> str:
    import mknodes as mk

    return str(node) if type(node) in {mk.MkText, mk.MkHeader} else node


def get_nondefault_repr(instance: object) -> str:
    import inspect

    spec = inspect.getfullargspec(instance.__init__)
    spec.args.remove("self")
    args = [getattr(instance, "items" if arg == "content" else arg) for arg in spec.args]
    dct = spec.kwonlydefaults or {}
    kwargs = {
        k: getattr(instance, k)
        for k, v in dct.items()
        if k in instance.__dict__ and v != getattr(instance, k)
    }
    return get_repr(instance, *args, **kwargs)


if __name__ == "__main__":
    import mknodes as mk

    instance = mk.MkAdmonition("test", typ="info")
    print(get_nondefault_repr(instance))



from __future__ import annotations

from mknodes.basenodes import mktabs, mktabcontainer
from mknodes.utils import log, resources


logger = log.get_logger(__name__)


class MkTabbedBlocks(mktabcontainer.MkTabContainer):
    """PyMdown Block Extension Tab."""

    items: list[mktabs.MkTabBlock]
    REQUIRED_EXTENSIONS = [resources.Extension("pymdownx.blocks.tab")]
    Tab = mktabs.MkTabBlock

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        # this one is basically the same as MkTabbed,
        # but based on new pymdownx block syntax.
        node = MkTabbedBlocks(tabs={"Tab 1": "Some markdown", "Tab 2": "Other Markdown"})
        page += mk.MkReprRawRendered(node, header="### Regular")


if __name__ == "__main__":
    tabs = dict(Tab1="Some text", Tab2="Another text")
    tabblock = MkTabbedBlocks(tabs)
    print(tabblock)



from __future__ import annotations

from mknodes.pages import pagetemplate


class TemplateRegistry:
    def __init__(self):
        self.templates: dict[str, pagetemplate.PageTemplate] = {}

    def __getitem__(self, value: str) -> pagetemplate.PageTemplate:
        return self.templates.setdefault(
            value,
            pagetemplate.PageTemplate(filename=value),
        )

    def __setitem__(self, index, value):
        self.templates[index] = value

    def __iter__(self):
        return iter(self.templates.values())

    def __len__(self):
        return len(self.templates)


if __name__ == "__main__":
    registry = TemplateRegistry()
    a = registry["main.html"]
    for template in registry:
        print(template)



from __future__ import annotations

import os

from typing import TYPE_CHECKING, Literal

from mknodes.basenodes import mknode
from mknodes.mdlib import mdconverter
from mknodes.utils import css as css_, helpers, resources


if TYPE_CHECKING:
    import markdown

    import mknodes as mk


BlockStr = Literal[
    "analytics",
    "announce",
    "config",
    "container",
    "content",
    "extrahead",
    "fonts",
    "footer",
    "header",
    "hero",
    "htmltitle",
    "libs",
    "outdated",
    "scripts",
    "site_meta",
    "site_nav",
    "styles",
    "tabs",
]


SUPER_TEXT = r"{{ super() }}"

REDIRECT = """
<link rel="canonical" href="{url}">
<meta name="robots" content="noindex">
<script>var anchor=window.location.hash.substr(1);location.href="{url}"+(anchor?"#"+anchor:"")</script>
<meta http-equiv="refresh" content="0; url={url}">
"""  # noqa: E501


class Super:
    """Simple class to use for the jinja super expression."""

    def __str__(self):
        return SUPER_TEXT


class BaseBlock(mknode.MkNode):
    """A class representing a block from a page template."""

    block_id: str

    def block_content(self, md: markdown.Markdown | None = None):
        raise NotImplementedError

    def to_markdown(self, md: markdown.Markdown | None = None):
        """Return HTML for the block.

        Arguments:
            md: Markdown parser to use
        """
        instance = md or mdconverter.MdConverter()
        content = self.block_content(instance)
        return f"{{% block {self.block_id} %}}\n{content}\n{{% endblock %}}"


class HtmlBlock(BaseBlock):
    """Base class for blocks which usually contain HTML content."""

    def __init__(self, block_id: str, parent: mk.MkPage | mk.MkNav | None = None):
        """Constructor.

        Arguments:
            block_id: Name of the block
            parent: Parent node
        """
        super().__init__(parent=parent)
        self.block_id = block_id
        self.items = [Super()]

    def __bool__(self):
        return len(self.items) != 1 or not isinstance(self.items[0], Super)

    def block_content(self, md: markdown.Markdown | None = None) -> str:
        """Return the actual block content.

        Arguments:
            md: Markdown parser to use
        """
        import mknodes as mk

        instance = md or mdconverter.MdConverter()
        result = ""
        for i in self.items:
            match i:
                case mk.MkNode():
                    i.parent = self.parent
                    result += instance.convert(str(i))
                case _:
                    result += str(i)
            result += "\n"
        return result

    @property
    def content(self):
        return self.items[0]

    @content.setter
    def content(self, value):
        value = self.to_child_node(value)
        self.items = [value]


class Block(BaseBlock):
    """Base class for head blocks."""

    def __init__(self):
        self.content = ""

    def __bool__(self):
        return bool(self.block_content())

    def block_content(self, md: markdown.Markdown | None = None):
        return self.content


class TitleBlock(BaseBlock):
    """Block wrapping the HTML title."""

    block_id = "htmltitle"

    def __init__(self, content: str | None = None):
        self.content = content

    def __bool__(self):
        return bool(self.content)

    def block_content(self, md: markdown.Markdown | None = None):
        return f"<title>{self.content}</title>"


class BaseJSBlock(BaseBlock):
    """Block for additional libraries."""

    def __init__(
        self,
        scripts: list[resources.JSFile] | None = None,
        include_super: bool = True,
    ):
        """Constructor.

        Arguments:
            scripts: List of scripts to add to HTML header libs.
            include_super: Whether to include the original content.
        """
        self.include_super = include_super
        self.scripts = scripts or []

    def add_script_file(self, script: resources.JSFile | str | os.PathLike):
        """Add a script file to the block.

        Arguments:
            script: Script to add to the block
        """
        if isinstance(script, str | os.PathLike):
            script = resources.JSFile(str(script))
        self.scripts.append(script)

    def __bool__(self):
        return bool(self.scripts)

    def block_content(self, md: markdown.Markdown | None = None):
        lines = [i.to_html() for i in self.scripts]
        scripts = "\n".join(lines)
        return f"{SUPER_TEXT}\n{scripts}" if self.include_super else scripts


class LibsBlock(BaseJSBlock):
    """Block for additional libraries."""

    block_id = "libs"


class ScriptsBlock(BaseJSBlock):
    """Block for JavaScripts at end of BODY."""

    block_id = "scripts"


class AnalyticsBlock(Block):
    """Block for analytics-tags in HEAD."""

    block_id = "analytics"


class SiteMetaBlock(Block):
    """Block for meta tags in HEAD."""

    block_id = "site_meta"


class ExtraHeadBlock(Block):
    """Block for extra HEAD content.

    Can be used for stuff not covered by other blocks, like changing the
    indexing rules.
    """

    block_id = "extrahead"

    def __init__(self):
        super().__init__()
        self.robots_rule = None
        self.redirect_url = None

    def block_content(self, md: markdown.Markdown | None = None):
        content = self.content
        if self.robots_rule:
            rule = f'\n<meta name="robots" content="{self.robots_rule}">'
            content += rule
        if self.redirect_url:
            content += REDIRECT.format(url=self.redirect_url)
        return content

    def set_robots_rule(self, rule: str | None = None):
        """Set a rule for search robots.

        For valid rule values, check
        https://developers.google.com/search/docs/crawling-indexing/robots-meta-tag

        Arguments:
            rule: Rule to apply for robots
        """
        self.robots_rule = rule

    def set_redirect_url(self, url: str):
        """Set a URL which the page should redirect to.

        Arguments:
            url: URL to redirect to
        """
        self.redirect_url = url


class StylesBlock(BaseBlock):
    """Block for additional stylesheets."""

    block_id = "styles"

    def __init__(
        self,
        styles: list[resources.CSSFile | resources.CSSText] | None = None,
        include_super: bool = True,
    ):
        self.include_super = include_super
        self.styles = styles or []

    def add_stylesheet(self, stylesheet: resources.CSSFile):
        self.styles.append(stylesheet)

    def add_css(self, css: str | dict):
        if isinstance(css, dict):
            css = str(css_.CSS(css))
        filename = f"{helpers.get_hash(css)}.css"
        text = resources.CSSText(filename=filename, content=css)
        self.styles.append(text)

    def __bool__(self):
        return bool(self.styles)

    def block_content(self, md: markdown.Markdown | None = None):
        lines = [i.to_html() for i in self.styles]
        styles = "\n".join(lines)
        return f"{SUPER_TEXT}\n{styles}" if self.include_super else styles


if __name__ == "__main__":
    block = StylesBlock()
    print(block.to_markdown())



from __future__ import annotations

from typing import TYPE_CHECKING, Any, get_args

from mknodes.basenodes import mkblock
from mknodes.data import datatypes
from mknodes.utils import log, reprhelpers, resources


if TYPE_CHECKING:
    import mknodes as mk

logger = log.get_logger(__name__)


class MkAdmonitionBlock(mkblock.MkBlock):
    """Pymdownx-based Admonition Block."""

    ICON = "octicons/info-16"
    REQUIRED_EXTENSIONS = [resources.Extension("pymdownx.blocks.admonition")]

    def __init__(
        self,
        content: str | list | mk.MkNode,
        *,
        typ: datatypes.AdmonitionTypeStr = "info",
        title: str | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            content: Admonition content
            typ: Admonition type
            title: Optional Admonition title
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(
            "admonition",
            content=content,
            argument=title or "",
            attributes=dict(type=typ),
            **kwargs,
        )
        self.typ = typ
        self.title = title

    def __repr__(self):
        if len(self.items) == 1:
            content = reprhelpers.to_str_if_textnode(self.items[0])
        else:
            content = [reprhelpers.to_str_if_textnode(i) for i in self.items]
        return reprhelpers.get_repr(
            self,
            content=content,
            typ=self.typ,
            title=self.title,
            _filter_empty=True,
        )

    @property
    def typ(self) -> datatypes.AdmonitionTypeStr:
        return self.attributes["type"]

    @typ.setter
    def typ(self, value: datatypes.AdmonitionTypeStr):
        self.attributes["type"] = value

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        page += "MkAdmonitionBlock is an admonition based on new pymdownx block syntax."
        url = "https://facelessuser.github.io/pymdown-extensions/extensions/blocks/api/"
        page += mk.MkLink(url, "More info", as_button=True)
        # AdmonitionTypeStr is a Literal containing all Admonition types
        for typ in get_args(datatypes.AdmonitionTypeStr):
            page += mk.MkHeader(f"Type '{typ}'", level=3)
            title = f"Block admonition with type {typ!r}"
            content = f"This is type **{typ}**"
            node = mk.MkAdmonitionBlock(typ=typ, content=content, title=title)
            page += mk.MkReprRawRendered(node)


if __name__ == "__main__":
    tab = MkAdmonitionBlock(content="test", title="test")
    print(tab)



from __future__ import annotations

import dataclasses

from mknodes.utils import log


logger = log.get_logger(__name__)


@dataclasses.dataclass
class EventPlugin:
    flow: list[str]
    help_link: str
    hook_fn_path: str
    entry_point_group: str


MKDOCS_FLOW = [
    "on_startup",
    "on_shutdown",
    "on_serve",
    "on_config",
    "on_pre_build",
    "on_files",
    "on_nav",
    "on_env",
    "on_post_build",
    "on_build_error",
    "on_pre_template",
    "on_template_context",
    "on_post_template",
    "on_pre_page",
    "on_page_read_source",
    "on_page_markdown",
    "on_page_content",
    "on_page_context",
    "on_post_page",
    "on_post_build",
    "on_serve",
    "on_shutdown",
]


mkdocs_plugin = EventPlugin(
    flow=MKDOCS_FLOW,
    help_link="https://www.mkdocs.org/dev-guide/plugins/#{event}",
    hook_fn_path="mkdocs.plugins.BasePlugin.{event}",
    entry_point_group="mkdocs.plugins",
)



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mklist
from mknodes.data import commitconventions
from mknodes.templatenodes import mkjinjatemplate
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


STYLES = {
    "Angular Style": "https://gist.github.com/stephenparish/9941e89d80e2bc58a153",
    "Karma convention": "https://karma-runner.github.io/4.0/dev/git-commit-msg.html",
}


class MkCommitConventions(mkjinjatemplate.MkJinjaTemplate):
    """Text node containing Commit message conventions."""

    ICON = "simple/conventionalcommits"
    STATUS = "new"

    def __init__(
        self,
        commit_types: (
            list[commitconventions.CommitTypeStr]
            | commitconventions.ConventionTypeStr
            | None
        ) = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            commit_types: Allowed commit commit_types. Can be "basic",
                          "conventional_commits", or a list of commit_types
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(template="commit_conventions.jinja", **kwargs)
        self._commit_types = commit_types

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            commit_types=self._commit_types,
            _filter_empty=True,
        )

    @property
    def variables(self):
        styles = " or ".join(f"[{k}]({v})" for k, v in STYLES.items())
        all_types = commitconventions.TYPE_DESCRIPTIONS
        items = [f"`{k}`: {all_types[k]}" for k in self.commit_types]
        ls = mklist.MkList(items)
        return dict(styles=styles, commit_types=str(ls))

    @variables.setter
    def variables(self, value):
        pass

    @property
    def commit_types(self) -> list[commitconventions.CommitTypeStr]:
        val: list[commitconventions.CommitTypeStr] | commitconventions.ConventionTypeStr
        match self._commit_types:
            case None:
                val = self.ctx.metadata.commit_types or "conventional_commits"
            case _:
                val = self._commit_types
        match val:
            case "basic":
                return list(commitconventions.basic.types)
            case "conventional_commits" | "angular" | None:
                return list(commitconventions.conventional_commits.types)
            case list():
                return val
            case _:
                raise TypeError(self._commit_types)

    @commit_types.setter
    def commit_types(self, value):
        self._commit_types = value

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkCommitConventions(header="")
        page += mk.MkReprRawRendered(node, header="### All commit_types")
        node = MkCommitConventions(["fix", "feat", "refactor"], header="")
        page += mk.MkReprRawRendered(node, header="### Selected commit_types")


if __name__ == "__main__":
    conventions = MkCommitConventions()
    print(conventions)



from __future__ import annotations

import pytest

import mknodes as mk


TREE_NUM_PAGES = 5
TREE_NUM_NAVS = 3
TREE_NUM_TEXT = 5
TREE_TOTAL = TREE_NUM_PAGES + TREE_NUM_NAVS + TREE_NUM_TEXT


def test_nav():
    nav = mk.MkNav()
    nav.add_doc(mk)
    nav.add_page("Test")
    nav.add_page("Test Index page", is_index=True)
    nav.add_nav("sub")


def test_from_folder(test_data_dir):
    nav = mk.MkNav()
    nav.parse.folder(test_data_dir / "nav_tree")
    assert len(list(nav.descendants)) == TREE_TOTAL - 1


def test_from_file(test_data_dir):
    nav_file = test_data_dir / "nav_tree/SUMMARY.md"
    nav = mk.MkNav()
    nav.parse.file(nav_file)
    assert len(list(nav.descendants)) == TREE_TOTAL - 1


def test_resolved_path():
    nav = mk.MkNav()
    subnav = nav.add_nav("subsection")
    subsubnav = subnav.add_nav("subsubsection")
    assert subsubnav.resolved_parts == ("subsection", "subsubsection")


def test_creating_module_document():
    nav = mk.MkNav()
    subnav = nav.add_nav("subsection")
    module_docs = subnav.add_doc(pytest)
    klasses = list(module_docs.iter_classes())
    assert klasses



from __future__ import annotations

import os
import pathlib

from typing import Any, Literal, get_args

import seedir

from mknodes.basenodes import mkcode
from mknodes.utils import log, reprhelpers, resources


logger = log.get_logger(__name__)


DirectoryTreeStyleStr = Literal["lines", "dash", "arrow", "spaces", "plus"]


class MkSeeDir(mkcode.MkCode):
    """Node to display directory content as a tree.

    Based on "seedir" package

    """

    ICON = "material/file-tree-outline"
    REQUIRED_PACKAGES = [resources.Package("seedir")]

    def __init__(
        self,
        directory: str | os.PathLike,
        *,
        style: DirectoryTreeStyleStr | None = None,
        indent: int = 4,
        depth_limit: int | None = None,
        item_limit: int | None = None,
        beyond: Literal["ellipsis", "content"] = "ellipsis",
        first: Literal["files", "folders"] | None = None,
        sort: bool = False,
        exclude_folders: list[str] | str | None = None,
        header: str = "",
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            directory: Folder path to prettyprint content from
            style: Print style
            indent: Specifies the amount of indentation added for each nesting level
            depth_limit: Maximum nesting depth to print
            item_limit: Maximum amount of items to print
            beyond: String to indicate directory contents beyond the limits.
            first: What to print first
            sort: Whether to sort the output
            exclude_folders: Folders to exclude from listing
            header: Section header
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(header, **kwargs)
        self.directory = pathlib.Path(directory)
        self.style = style or "lines"
        self.print_indent = indent
        self.depth_limit = depth_limit
        self.item_limit = item_limit
        self.beyond = beyond
        self.first = first
        self.sort = sort
        self.exclude_folders = exclude_folders

    @property
    def text(self):
        return seedir.seedir(
            self.directory,
            style=self.style,
            printout=False,
            indent=self.print_indent,
            depthlimit=self.depth_limit,
            itemlimit=self.item_limit,
            beyond=self.beyond,
            first=self.first,
            sort=self.sort,
            exclude_folders=self.exclude_folders,
        )

    @text.setter
    def text(self, text):
        self.obj = text

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            path=self.directory,
            style=self.style,
            indent=self.print_indent,
            depth_limit=self.depth_limit,
            item_limit=self.item_limit,
            beyond=self.beyond,
            first=self.first,
            sort=self.sort,
            exclude_folders=self.exclude_folders,
            _filter_empty=True,
            _filter_false=True,
        )

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        for style in get_args(DirectoryTreeStyleStr):
            node = MkSeeDir("mknodes/manual", style=style)
            page += mk.MkReprRawRendered(node, header=f"### Style '{style}'")


if __name__ == "__main__":
    node = MkSeeDir(".", header="test", style="dash")
    print(node.to_markdown())



from __future__ import annotations

import inspect
import os
import pathlib
import textwrap
import types

from typing import TYPE_CHECKING, Any, Self

from mknodes.basenodes import mkcontainer
from mknodes.data import datatypes
from mknodes.utils import classhelpers, inspecthelpers, log, reprhelpers, resources


if TYPE_CHECKING:
    import mknodes as mk

logger = log.get_logger(__name__)


class MkCode(mkcontainer.MkContainer):
    """Class representing a Code block."""

    ICON = "material/code-json"
    REQUIRED_EXTENSIONS = [
        resources.Extension(
            "pymdownx.highlight",
            anchor_linenums=True,
            line_spans="__span",
            pygments_lang_class=True,
        ),
        resources.Extension("pymdownx.superfences"),
    ]

    def __init__(
        self,
        code: str | mk.MkNode | list = "",
        language: str = "py",
        *,
        title: str = "",
        linenums: int | None = None,
        highlight_lines: list[int] | None = None,
        fence_level: int | None = None,
        header: str = "",
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            code: Code to show
            language: language for syntax highlighting
            title: Code block title
            linenums: If set, use as start linenumber
            highlight_lines: Optionally highlight lines
            fence_level: Determines amount of ticks used for fence.
                         If None, auto-determine based on nesting depth.
            header: Section header
            kwargs: Keyword arguments passed to parent
        """
        self.language = language
        self.title = title
        self.linenums = linenums
        self.highlight_lines = highlight_lines
        self._fence_level = fence_level
        super().__init__(content=code, header=header, **kwargs)

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            code=self.items,
            language=self.language,
            title=self.title,
            linenums=self.linenums,
            highlight_lines=self.highlight_lines,
            _filter_empty=True,
        )

    @property
    def text(self) -> str:
        """Text content."""
        return "\n".join(str(i) for i in self.items)

    @property
    def fence_boundary(self) -> str:
        """Return fence boundary, based on nesting level. Default is ```."""
        if self._fence_level:
            return "`" * (self._fence_level + 3)
        block_level = sum(isinstance(i, MkCode) for i in self.ancestors)
        return "`" * (block_level + 3)

    @property
    def fence_title(self) -> str:
        """Title in first line."""
        title = f" title={self.title!r}" if self.title else ""
        if self.linenums is not None:
            title += f' linenums="{self.linenums}"'
        if self.highlight_lines:
            title += ' hl_lines="' + " ".join(str(i) for i in self.highlight_lines) + '"'
        return f"{self.language}{title}"

    def _to_markdown(self) -> str:
        first_line = f"{self.fence_boundary} {self.fence_title}"
        return f"{first_line}\n{self.text}\n{self.fence_boundary}"

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        page += mk.MkHeader("Regular", level=3)
        node_1 = MkCode(language="python", code="a = 1 + 2")
        page += mk.MkReprRawRendered(node_1)

        page += mk.MkHeader("Syntax highlighting", level=3)
        node_2 = MkCode(language="js", code="var z = x + y;", title="syntax highlight")
        page += mk.MkReprRawRendered(node_2)

        page += mk.MkHeader("Highlighting lines", level=3)
        node_3 = MkCode(code="1\n2\n3\n4", highlight_lines=[1, 3])
        page += mk.MkReprRawRendered(node_3)

        page += mk.MkHeader("Numbered lines", level=3)
        node_4 = MkCode(code="1\n2\n3\n4", linenums=10)
        page += mk.MkReprRawRendered(node_4)

    @classmethod
    def for_file(
        cls,
        path: str | os.PathLike,
        *,
        linenums: bool = True,
        highlight_caller: bool = True,
        title: str | None = None,
        language: str | None = None,
        **kwargs: Any,
    ):
        """Create a MkCode node based on a code file.

        Line numbers will be shown by default. If `highlight_caller` is `True`,
        it will try to detect whether the calling method is inside the code block are
        creating and if yes, it will highlight that line.

        Arguments:
            path: Path to the code file
            linenums: Whether to show line numbers
            highlight_caller: Whether we want to try to highlight the line which called
                              this method.
            title: title to use for the code box. If None is set, filename will be used.
            language: Syntax highlighting language. If None, try to infer from extension.
            kwargs: Keyword arguments passed to MkCode ctor
        """
        path = pathlib.Path(path)
        with path.open() as file:
            content = file.read()
        hl_lines = None
        if highlight_caller and (frame := inspect.currentframe()) and frame.f_back:
            call_file = frame.f_back.f_code.co_filename
            if call_file == str(path.absolute()):
                line_count = content.count("\n")
                line = frame.f_back.f_lineno
                hl_lines = [line] if 0 <= line <= line_count else None
        start_line = 1 if linenums else None
        title = path.name if title is None else title
        if language is None:
            language = datatypes.EXT_TO_PYGMENTS_STYLE.get(path.suffix, "")
        return cls(
            content,
            linenums=start_line,
            title=title,
            highlight_lines=hl_lines,
            language=language,
            **kwargs,
        )

    @classmethod
    def for_object(
        cls,
        obj: datatypes.HasCodeType,
        *,
        dedent: bool = True,
        extract_body: bool = False,
        title: str | None = None,
        linenums: bool = True,
        highlight_caller: bool = True,
        **kwargs: Any,
    ) -> Self:
        """Create a MkCode node based on a python object.

        Fetches code by using the inspect module.
        Line numbers will be shown by default. If highlight_caller is True,
        it will try to detect whether the calling method is inside the code block are
        displaying and if yes, it will highlight that line.

        Arguments:
            obj: Python object to show code from
            dedent: Whether to dedent the code
            extract_body: if True, Function / Class signatures are stripped from the code
            title: Title to use for code block. If None, it will use the object path.
            linenums: Whether to show line numbers
            highlight_caller: Whether we want to try to highlight the line which called
                              this method.
            kwargs: Keyword arguments passed to MkCode ctor
        """
        if extract_body and callable(obj):
            code = inspecthelpers.get_function_body(obj)
        elif extract_body:
            msg = "Can only extract body from Functions, Methods and classes"
            raise TypeError(msg)
        else:
            code = inspecthelpers.get_source(obj)
        code = textwrap.dedent(code) if dedent else code
        code_title = title if title is not None else classhelpers.get_code_name(obj)
        hl_lines = None
        lines, start_line = inspecthelpers.get_source_lines(obj)
        if isinstance(obj, types.ModuleType):
            start_line += 1
        if highlight_caller and (frame := inspect.currentframe()) and frame.f_back:
            call_file = frame.f_back.f_code.co_filename
            obj_file = inspect.getfile(obj)
            if call_file == obj_file:
                line_no = frame.f_back.f_lineno
                line = line_no - start_line + 1
                hl_lines = [line] if 0 <= line <= start_line + len(lines) else None
        return cls(
            code=code,
            title=code_title,
            linenums=start_line if linenums else None,
            highlight_lines=hl_lines,
            **kwargs,
        )


if __name__ == "__main__":
    code = MkCode.for_object(MkCode, extract_body=True)
    code2 = MkCode(code)
    print(code2)



from __future__ import annotations

from collections.abc import Callable, Iterator, Sequence
import inspect
import types

from typing import Any

from mknodes.navs import mknav
from mknodes.pages import mkclasspage, mkmodulepage
from mknodes.utils import classhelpers, log, reprhelpers


logger = log.get_logger(__name__)


class MkDoc(mknav.MkNav):
    """Nav for showing a module documenation."""

    def __init__(
        self,
        module: types.ModuleType | Sequence[str] | str | None = None,
        *,
        filter_by___all__: bool = False,
        exclude_modules: list[str] | None = None,
        section_name: str | None = None,
        recursive: bool = False,
        class_page: type[mkclasspage.MkClassPage] | str | None = None,
        module_page: type[mkmodulepage.MkModulePage] | str | None = None,
        flatten_nav: bool = False,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            module: Module to document
            filter_by___all__: Whether to filter stuff according to "__all__"
            recursive: Whether to search modules recursively
            exclude_modules: List of modules to exclude
            section_name: Optional section name override
            class_page: Override for the default ClassPage
            module_page: Override for the default ModulePage
            flatten_nav: Whether classes should be put into top-level of the nav
            kwargs: Keyword arguments passed to parent
        """
        if module:
            self._module = classhelpers.to_module(module, return_none=False)
        else:
            self._module = None
        self.ClassPage = class_page or mkclasspage.MkClassPage
        self.ModulePage = module_page or mkmodulepage.MkModulePage
        self.flatten_nav = flatten_nav
        self.recursive = recursive
        self.klasses: set[type] = set()
        self.submodules: set[types.ModuleType] = set()
        self.filter_by___all__ = filter_by___all__
        self._exclude = exclude_modules or []
        # self.root_path = pathlib.Path(f"./{self.module_name}")
        super().__init__(section=section_name or self.module_name, **kwargs)
        self._collect_classes()

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            module=self.module_name,
            section=self.section or "<root>",
            filename=self.filename,
        )

    @property
    def module(self):
        return self.ctx.metadata.module if self._module is None else self._module

    @property
    def module_name(self) -> str:
        return self.module.__name__.split(".")[-1]

    def _collect_classes(self):
        """Collect classes from given module."""
        for klass in self.iter_classes(recursive=self.recursive):
            self.klasses.add(klass)
        for klass in self.klasses:
            self.add_class_page(klass=klass, flatten=self.flatten_nav)
        for submod in self.submodules:
            self.add_doc(
                submod,
                class_page=self.ClassPage,
                filter_by___all__=self.filter_by___all__,
                module_page=self.ModulePage,
                flatten_nav=self.flatten_nav,
            )
        self._create_index_page()

    def iter_classes(
        self,
        submodule: types.ModuleType | str | tuple | list | None = None,
        *,
        recursive: bool = False,
        predicate: Callable[[type], bool] | None = None,
        _seen: set | None = None,
    ) -> Iterator[type]:
        """Iterate over all classes of the module.

        Arguments:
            submodule: filter based on a submodule
            recursive: whether to only iterate over members of current module
                       or whether it should also include classes from submodules.
            predicate: filter classes based on a predicate.
        """
        if isinstance(submodule, list):
            submodule = tuple(submodule)
        mod = classhelpers.to_module(submodule) if submodule else self.module
        if mod is None:
            return
        if recursive:
            seen = _seen or set()
            for _submod_name, submod in inspect.getmembers(mod, inspect.ismodule):
                if submod.__name__.startswith(self.module_name) and submod not in seen:
                    seen.add(submod)
                    yield from self.iter_classes(
                        submod,
                        recursive=True,
                        predicate=predicate,
                        _seen=seen,
                    )
        for klass_name, klass in inspect.getmembers(mod, inspect.isclass):
            if self.filter_by___all__ and (
                not hasattr(mod, "__all__") or klass_name not in mod.__all__
            ):
                continue
            if predicate and not predicate(klass):
                continue
            # if klass.__module__.startswith(self.module_name):
            if self.module_name in klass.__module__.split("."):
                yield klass

    def add_class_page(
        self,
        klass: type,
        *,
        find_topmost: bool = True,
        flatten: bool = False,
        **kwargs: Any,
    ) -> mkclasspage.MkClassPage:
        """Add a page showing information about a class.

        Arguments:
            klass: klass to build a page for
            find_topmost: Whether to use a module path from a parent package if available
            flatten: Put page into top level nav if nested.
            kwargs: keyword arguments passed to CLassPage
        """
        if find_topmost:
            parts = classhelpers.get_topmost_module_path(klass).split(".")
        else:
            parts = klass.__module__.split(".")
        if isinstance(self.ClassPage, str):
            page = mkclasspage.MkClassPage(
                klass=klass,
                template=self.ClassPage,
                module_path=tuple(parts),
                parent=self,
                **kwargs,
            )
        else:
            page = self.ClassPage(
                klass=klass,
                module_path=tuple(parts),
                parent=self,
                **kwargs,
            )
        section = (klass.__name__,) if flatten else (*parts[1:], klass.__name__)
        self.nav[section] = page
        return page

    def _create_index_page(
        self,
        title: str | None = None,
        **kwargs: Any,
    ) -> mkmodulepage.MkModulePage:
        """Add a page showing all submodules.

        Arguments:
            title: Override title for the section.
            kwargs: kwargs passed to MkModulePage.
        """
        path = "index.md" if title is None else f"{title}.md"
        if isinstance(self.ModulePage, str):
            page = mkmodulepage.MkModulePage(
                module=self.module,
                klasses=self.klasses,
                template_name=self.ModulePage,
                path=path,
                parent=self,
                **kwargs,
            )
        else:
            page = self.ModulePage(
                module=self.module,
                klasses=self.klasses,
                path=path,
                parent=self,
                **kwargs,
            )
        self.index_page = page
        self.index_title = title or self.module_name
        return page


if __name__ == "__main__":
    doc = MkDoc(module="mkdocs")
    page = doc.add_class_page(MkDoc)
    print(page)



from __future__ import annotations

import mknodes as mk


def test_page():
    page = mk.MkPage()
    page.metadata.pop("created")
    assert not str(page)


EXPECTED = """---
description: Some description
hide:
- toc
- path
icon: material/emoticon-happy
search:
  boost: 2.0
  exclude: false
status: new
subtitle: Some subtitle
title: Some title
---

"""


def test_metadata():
    page = mk.MkPage(
        hide="toc,path",
        search_boost=2.0,
        exclude_from_search=False,
        icon="material/emoticon-happy",
        status="new",
        title="Some title",
        subtitle="Some subtitle",
        description="Some description",
    )
    page.metadata.pop("created")
    assert str(page) == EXPECTED



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mknode
from mknodes.utils import log, reprhelpers, resources, xmlhelpers as xml


logger = log.get_logger(__name__)


JS_URL = "https://unpkg.com/img-comparison-slider@7/dist/index.js"

CSS_URL = "https://unpkg.com/img-comparison-slider@7/dist/styles.css"


class ImgComparisonSlider(xml.HTMLElement):
    tag_name = "img-comparison-slider"


class MkImageCompare(mknode.MkNode):
    """Node to show an Image comparison (using a slider)."""

    ICON = "material/image-off"
    JS_FILES = [resources.JSFile(JS_URL, defer=True)]
    CSS = [resources.CSSFile(CSS_URL)]

    def __init__(
        self,
        before_image: str,
        after_image: str,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            before_image: Image shown with slider to the right
            after_image: Image shown with slider to the left
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self.before_image = before_image
        self.after_image = after_image

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            _filter_empty=True,
            _filter_false=True,
        )

    def get_element(self) -> ImgComparisonSlider:
        root = ImgComparisonSlider()
        xml.Img(src=self.before_image, slot="first", parent=root)
        xml.Img(src=self.after_image, slot="second", parent=root)
        return root

    def _to_markdown(self) -> str:
        root = self.get_element()
        return root.to_string()

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkImageCompare(
            before_image="https://picsum.photos/700",
            after_image="https://picsum.photos/701",
        )
        page += mk.MkReprRawRendered(node)


if __name__ == "__main__":
    img = MkImageCompare("a", "b")



from __future__ import annotations

import io
import pathlib

import fsspec
import jinja2

from mknodes.utils import log


logger = log.get_logger(__name__)


class JinjaLoaderFileSystem(fsspec.AbstractFileSystem):
    """A **FsSpec** Filesystem implementation for jinja environment templates.

    This virtual file system allows to browse and access all available templates of an
    environment by utilizing `BaseLoader.list_templates` and `BaseLoader.get_source`.
    """

    protocol = "jinja"

    def __init__(self, env: jinja2.Environment):
        super().__init__()
        self.env = env

    def ls(self, path: str, detail: bool = True, **kwargs) -> list[str]:
        if not self.env.loader:
            return []
        paths = self.env.loader.list_templates()
        if not path:
            return paths
        path = pathlib.Path(path).as_posix().rstrip("/")
        folders = [i for i in paths if i.rsplit("/", 1)[0] == path]
        if not folders:
            raise FileNotFoundError(path)
        return folders

    def _open(self, path: str, mode="rb", **kwargs) -> io.BytesIO:
        if not self.env.loader:
            msg = "Environment has no loader set"
            raise RuntimeError(msg)
        src, _filename, _uptodate = self.env.loader.get_source(self.env, path)
        return io.BytesIO(src.encode())


if __name__ == "__main__":
    from mknodes.jinja import loaders as loaders_

    fsspec.register_implementation("jinja", JinjaLoaderFileSystem)
    env = jinja2.Environment(loader=loaders_.resource_loader)
    fs = fsspec.filesystem("jinja", env=env)
    # print(fs.ls(""))
    with fs.open("licenses/templates/TORQUE-1.1.txt") as file:
        print(file.read())



from __future__ import annotations

import textwrap

from mknodes.basenodes import mkcontainer
from mknodes.utils import log


logger = log.get_logger(__name__)


class MkBlockQuote(mkcontainer.MkContainer):
    """BlockQuote node."""

    ICON = "material/format-quote-open"
    STATUS = "new"

    def _to_markdown(self) -> str:
        text = super()._to_markdown()
        return textwrap.indent(text, "> ").rstrip("\n") + "\n"

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        page += "An MkBlockQuote can display > Blockquotes."
        block = MkBlockQuote(content="Some text")
        page += mk.MkReprRawRendered(block, header="### Regular")
        # We can also nest blocks, they will adjust their delimiters automatically.
        nested_quote = MkBlockQuote(MkBlockQuote("nested"))
        page += mk.MkReprRawRendered(nested_quote, header="### Nested")


if __name__ == "__main__":
    inner_1 = MkBlockQuote("inner_1\nmultiline")
    inner_2 = MkBlockQuote("inner_2")
    outer = MkBlockQuote(content=[inner_1, inner_2, "Not nested"])
    print(outer)



from __future__ import annotations

from collections.abc import Mapping
import functools
import os
import pathlib
import posixpath
import shutil

from mknodes.utils import log


logger = log.get_logger(__name__)


def copy(
    source_path: str | os.PathLike,
    output_path: str | os.PathLike,
    exist_ok: bool = True,
):
    """Copy source_path to output_path, making sure any parent directories exist.

    The output_path may be a directory.

    Arguments:
        source_path: File to copy
        output_path: path where file should get copied to.
        exist_ok: Whether exception should be raised in case stuff would get overwritten
    """
    output_path = pathlib.Path(output_path)
    source_path = pathlib.Path(source_path)
    output_path.parent.mkdir(parents=True, exist_ok=exist_ok)
    if source_path.is_dir():
        if output_path.is_dir():
            msg = "Cannot copy folder to file!"
            raise RuntimeError(msg)
        shutil.copytree(source_path, output_path, dirs_exist_ok=exist_ok)
    else:
        if output_path.is_dir():
            output_path = output_path / source_path.name
        shutil.copyfile(source_path, output_path)


def write_file(content: str | bytes, output_path: str | os.PathLike):
    """Write content to output_path, making sure any parent directories exist.

    Arguments:
        content: Content to write
        output_path: path where file should get written to.
    """
    output_path = pathlib.Path(output_path)
    output_path.parent.mkdir(parents=True, exist_ok=True)
    mode = "wb" if isinstance(content, bytes) else "w"
    encoding = None if "b" in mode else "utf-8"
    with output_path.open(mode=mode, encoding=encoding) as f:
        f.write(content)


def write_files(mapping: Mapping[str | os.PathLike, str | bytes]):
    """Write a mapping of filename-to-content to disk.

    Arguments:
        mapping: {"path/to/file.ext": b"content", ...} - style mapping
    """
    for k, v in mapping.items():
        write_file(v, k)


def find_file_in_folder_or_parent(
    filename: str | pathlib.Path,
    folder: os.PathLike | str = ".",
) -> pathlib.Path | None:
    """Search for a file with given name in folder and its parent folders.

    Arguments:
        filename: File to search
        folder: Folder to start searching from
    """
    path = pathlib.Path(folder).absolute()
    while not (path / filename).exists() and len(path.parts) > 1:
        path = path.parent
    return file if (file := (path / filename)).exists() else None


@functools.cache
def load_file_cached(path: str | os.PathLike) -> str:
    if "://" in str(path):
        return fsspec_get(str(path))
    return pathlib.Path(path).read_text(encoding="utf-8")


def download_from_github(
    org: str,
    repo: str,
    path: str | os.PathLike,
    destination: str | os.PathLike,
    username=None,
    token=None,
    recursive: bool = False,
):
    import fsspec

    token = token or os.environ.get("GITHUB_TOKEN")
    if token and not username:
        token = None
    destination = pathlib.Path(destination)
    destination.mkdir(exist_ok=True, parents=True)
    fs = fsspec.filesystem("github", org=org, repo=repo)
    logger.info("Copying files from Github: %s", path)
    fs.get(fs.ls(str(path)), destination.as_posix(), recursive=recursive)


@functools.cache
def fsspec_get(path: str) -> str:
    import fsspec

    with fsspec.open(path) as file:
        return file.read().decode()


@functools.lru_cache
def _get_norm_url(path: str) -> tuple[str, int]:
    from urllib.parse import urlsplit

    if not path:
        path = "."
    elif "\\" in path:
        logger.warning(
            "Path %r uses OS-specific separator '\\'. "
            "That will be unsupported in a future release. Please change it to '/'.",
            path,
        )
        path = path.replace("\\", "/")
    # Allow links to be fully qualified URLs
    parsed = urlsplit(path)
    if parsed.scheme or parsed.netloc or path.startswith(("/", "#")):
        return path, -1

    # Relative path - preserve information about it
    norm = posixpath.normpath(path) + "/"
    relative_level = 0
    while norm.startswith("../", relative_level * 3):
        relative_level += 1
    return path, relative_level


def normalize_url(path: str, url: str | None = None, base: str = "") -> str:
    """Return a URL relative to the given page or using the base."""
    path, relative_level = _get_norm_url(path)
    if relative_level == -1:
        return path
    if url is not None:
        result = relative_url(url, path)
        if relative_level > 0:
            result = "../" * relative_level + result
        return result

    return posixpath.join(base, path)


def relative_url(url_a: str, url_b: str) -> str:
    """Compute the relative path from URL A to URL B.

    Arguments:
        url_a: URL A.
        url_b: URL B.

    Returns:
        The relative URL to go from A to B.
    """
    parts_a = url_a.split("/")
    if "#" in url_b:
        url_b, anchor = url_b.split("#", 1)
    else:
        anchor = None
    parts_b = url_b.split("/")

    # remove common left parts
    while parts_a and parts_b and parts_a[0] == parts_b[0]:
        parts_a.pop(0)
        parts_b.pop(0)

    # go up as many times as remaining a parts' depth
    levels = len(parts_a) - 1
    parts_relative = [".."] * levels + parts_b
    relative = "/".join(parts_relative)
    return f"{relative}#{anchor}" if anchor else relative


if __name__ == "__main__":
    file = download_from_github("phil65", "mknodes", "mknodes", "testus/")
    print(file)



from __future__ import annotations

import coloraide


rgb_tuple_len = 3
ALPHA_DEFAULT = 1.0


class Color(coloraide.Color):
    def __init__(
        self,
        color: str | tuple,
        data=None,
        alpha: float = ALPHA_DEFAULT,
        **kwargs,
    ):
        match color:
            case str():
                super().__init__(color, data=data, alpha=alpha, **kwargs)
                if not data and alpha != ALPHA_DEFAULT:
                    self["alpha"] = alpha
            case tuple() if len(color) == rgb_tuple_len:
                super().__init__("srgb", [i / 255 for i in color], alpha=alpha, **kwargs)
            case tuple():
                super().__init__("srgb", [i / 255 for i in color[:3]], color[3], **kwargs)
            case _:
                raise TypeError(color)

    def __str__(self):
        """Return str in form of rgb(255, 0, 0)."""
        return self.to_string(comma=True)

    def brightness(self, value: float, **kwargs):
        return self.filter("brightness", value, **kwargs)

    def set_alpha(self, value: float):
        color["alpha"] = value


if __name__ == "__main__":
    color = Color("red", alpha=0.5)
    print(color)



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mkcontainer
from mknodes.data import tools
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


class MkDevTools(mkcontainer.MkContainer):
    """Node showing information about used dev tools."""

    ICON = "material/wrench"
    STATUS = "new"

    def __init__(
        self,
        tools: list[tools.Tool] | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            tools: Tools to show install / setup instructions for.
                            If None, tools will be pulled from project.
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self._tools = tools

    def __repr__(self):
        return reprhelpers.get_repr(self, tools=self._tools, _filter_empty=True)

    @property
    def tools(self) -> list[tools.Tool]:  # type: ignore[return]
        match self._tools:
            case list():
                return self._tools
            case None:
                return self.ctx.metadata.tools
            case _:
                raise TypeError(self._tools)

    @property
    def items(self):
        import mknodes as mk

        items = []
        for tool in self.tools:
            cfg_node = mk.MkCode(tool.cfg or "", language=tool.config_syntax)
            code = mk.MkCode(f"pip install {tool.identifier}", language="bash")
            link = mk.MkLink(tool.url, "More information")
            in_adm = [f"To install {tool.identifier}:", code, link]
            title = f"Installing {tool.title}"
            section = [
                mk.MkHeader(tool.title),
                mk.MkText(tool.description),
                mk.MkCode(tool.setup_cmd, language="md") if tool.setup_cmd else None,
                mk.MkAdmonition(cfg_node, collapsible=True, title="Config", typ="quote"),
                mk.MkAdmonition(in_adm, collapsible=True, title=title),
            ]
            items.extend(i for i in section if i is not None)
        for item in items:
            item.parent = self
        return items

    @items.setter
    def items(self, value):
        pass

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkDevTools()
        page += mk.MkReprRawRendered(node)


if __name__ == "__main__":
    setup_text = MkDevTools(build_backend="flit")
    print(setup_text)



from __future__ import annotations

from typing import Literal

from mknodes.info import folderinfo
from mknodes.utils import pathhelpers, yamlhelpers


PRE_COMMIT_CODE = """
# Setup pre-commit hooks for required formatting
pre-commit install
"""

PRE_COMMIT_TEXT = """This project uses **pre-commit** to ensure code quality.
A `.pre-commit-config.yaml` configuration file tailored for this project is provided
in the root folder."""


MYPY_CODE = """
{{metadata.build_system.run_prefix}}mypy --help
"""

MYPY_TEXT = """**MyPy** is used for type checking. You can find the configuration in the
`pyproject.toml` file."""

RUFF_CODE = """
{{metadata.build_system.run_prefix}}ruff --help
"""

RUFF_TEXT = """**Ruff** is used as a linter. You can find the configuration in the
`pyproject.toml` file."""

BLACK_CODE = """
{{metadata.build_system.run_prefix}}black .
"""

BLACK_TEXT = """**Black** is used as a code formatter. You can find the configuration in
the `pyproject.toml` file."""

COVERAGE_CODE = """
{{metadata.build_system.run_prefix}}coverage run some_module.py
"""

COVERAGE_TEXT = """**Coverage** is used to monitor test coverage."""

MKDOCS_CODE = """
# To build the docs
{{metadata.build_system.run_prefix}}mkdocs build

# To serve the docs locally at http://127.0.0.1:8000/
{{metadata.build_system.run_prefix}}mkdocs serve

# For additional mkdocs help and options:
{{metadata.build_system.run_prefix}}mkdocs --help
"""

MKDOCS_TEXT = """**MkDocs** is used to create the documentation."""


MATERIAL_TEXT = """**Material for MkDocs** is used as the Website theme."""


class Tool:
    identifier: ToolStr
    title: str
    url: str
    description: str
    setup_cmd: str | None
    config_syntax: str

    def __init__(self, folderinfo):
        self.used = self.is_used(folderinfo)
        self.cfg = self.get_config(folderinfo) if self.used else None

    def is_used(self, folder: folderinfo.FolderInfo) -> bool:
        """Return whether tool is used for given directory.

        Arguments:
            folder: Folder to check. Defaults to current working directory.
        """
        raise NotImplementedError

    def get_config(self, folder: folderinfo.FolderInfo) -> str | None:
        """Return config for given tool.

        Arguments:
            folder: Folder to get config from. Defaults to current working directory.
        """
        return None


class PreCommit(Tool):
    identifier = "pre-commit"
    title = "Pre-Commit"
    url = "https://pre-commit.com"
    description = PRE_COMMIT_TEXT
    setup_cmd = PRE_COMMIT_CODE
    config_syntax = "yaml"
    cfg_file = ".pre-commit-config.yaml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return bool(pathhelpers.find_file_in_folder_or_parent(self.cfg_file, folder.path))

    def get_config(self, folder):
        path = pathhelpers.find_file_in_folder_or_parent(self.cfg_file, folder.path)
        return path.read_text(encoding="utf-8") if path else None


class Ruff(Tool):
    identifier = "ruff"
    title = "Ruff"
    url = "https://beta.ruff.rs/"
    description = RUFF_TEXT
    setup_cmd = RUFF_CODE
    config_syntax = "toml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return "ruff" in folder.pyproject.tool

    def get_config(self, folder):
        return folder.pyproject.get_section_text("tool", "ruff")


class Black(Tool):
    identifier = "black"
    title = "Black"
    url = "https://github.com/psf/black"
    description = BLACK_TEXT
    setup_cmd = BLACK_CODE
    config_syntax = "toml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return "black" in folder.pyproject.tool

    def get_config(self, folder):
        return folder.pyproject.get_section_text("tool", "black")


class MyPy(Tool):
    identifier = "mypy"
    title = "MyPy"
    url = "https://mypy-lang.org"
    description = MYPY_TEXT
    setup_cmd = MYPY_CODE
    config_syntax = "toml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return "mypy" in folder.pyproject.tool

    def get_config(self, folder):
        return folder.pyproject.get_section_text("tool", "mypy")


class Coverage(Tool):
    identifier = "coverage"
    title = "Coverage"
    url = "https://coverage.readthedocs.io/"
    description = COVERAGE_TEXT
    setup_cmd = COVERAGE_CODE
    config_syntax = "toml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return (
            "coverage" in folder.pyproject.tool or (folder.path / ".coveragerc").exists()
        )

    def get_config(self, folder):
        text = folder.pyproject.get_section_text("tool", "coverage")
        if (path := (folder.path / ".coveragerc")).exists():
            return f"{text}\n{path.read_text(encoding='utf-8')}"
        return text


class MkDocs(Tool):
    identifier = "mkdocs"
    title = "MkDocs"
    url = "https://mkdocs.org/"
    description = MKDOCS_TEXT
    setup_cmd = MKDOCS_CODE
    config_syntax = "yaml"

    def is_used(self, folder: folderinfo.FolderInfo):
        return bool(folder.mkdocs_config)

    def get_config(self, folder):
        return folder.mkdocs_config.serialize("yaml")


class MkDocsMaterial(Tool):
    identifier = "mkdocs-material"
    title = "Material for MkDocs"
    url = "https://squidfunk.github.io/mkdocs-material/"
    description = MATERIAL_TEXT
    setup_cmd = None
    config_syntax = "yaml"

    def is_used(self, folder: folderinfo.FolderInfo):
        if (
            folder
            and folder.mkdocs_config
            and (theme := folder.mkdocs_config.get("theme"))
        ):
            return (
                theme == "material"
                if isinstance(theme, str)
                else theme.get("name") == "material"
            )
        return False

    def get_config(self, folder):
        return folder and yamlhelpers.dump_yaml(folder.mkdocs_config.get("theme"))


ToolStr = Literal[
    "pre-commit",
    "ruff",
    "mypy",
    "coverage",
    "mkdocs",
    "mkdocs-material",
    "black",
]


TOOLS: dict[ToolStr, type[Tool]] = {
    p.identifier: p
    for p in [
        PreCommit,
        Ruff,
        Black,
        MyPy,
        Coverage,
        MkDocs,
        MkDocsMaterial,
    ]
}



from __future__ import annotations

from collections.abc import Mapping
import functools
import os
import types

from typing import Any

from mknodes.basenodes import mknode
from mknodes.utils import inspecthelpers, log, reprhelpers, resources


logger = log.get_logger(__name__)


@functools.cache
def get_dependency_svg(
    folder_name: str | os.PathLike,
    max_bacon: int | None = 2,
    max_module_depth: int | None = None,
    only_cycles: bool = False,
    clusters: bool = False,
) -> str:
    """Get svg for given folder (like "src/module" or just "module")."""
    from pydeps import cli, colors, dot, py2depgraph
    from pydeps.pydeps import depgraph_to_dotsrc
    from pydeps.target import Target

    cli.verbose = cli._not_verbose
    cmd = [str(folder_name), "--noshow"]
    if max_bacon:
        cmd.extend(["--max-bacon", str(max_bacon)])
    if max_module_depth:
        cmd.extend(["--max-module-depth", str(max_module_depth)])
    if only_cycles:
        cmd.append("--only-cycles")
    if clusters:
        cmd.append("--clusters")
    options = cli.parse_args(cmd)
    colors.START_COLOR = options["start_color"]
    target = Target(options["fname"])
    logger.debug("run py2dep for folder %r", str(folder_name))
    with target.chdir_work():
        dep_graph = py2depgraph.py2dep(target, **options)
    logger.debug("run depgraph_to_dotsrc")
    dot_src = depgraph_to_dotsrc(target, dep_graph, **options)
    logger.debug("run call_graphviz_dot")
    svg = dot.call_graphviz_dot(dot_src, "svg").decode()
    svg = "".join(svg.splitlines()[6:])
    return svg.replace('fill="white"', 'fill="transparent"')


def insert_links(svg: str, link_map: Mapping[str, str]) -> str:
    for k, v in link_map.items():
        if (title_tag := f'<title>{k.replace(".", "_")}</title>') in svg:
            svg = svg.replace(title_tag, f'<a href="{v}"><title>{k}</title>')
    return svg.replace("</text></g>", "</text></a></g>")


class MkPyDeps(mknode.MkNode):
    """Node for showing a Dependency graph."""

    ICON = "material/code-json"
    REQUIRED_PACKAGES = [resources.Package("pydeps")]

    def __init__(
        self,
        module: str | os.PathLike | types.ModuleType | None = None,
        max_bacon: int | None = None,
        max_module_depth: int | None = None,
        only_cycles: bool = False,
        clusters: bool = False,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            module: Path to the module
            max_bacon: Max bacon
            max_module_depth: Maximum module depth to display
            only_cycles: Only show import cycles
            clusters: draw external dependencies as separate clusters
            kwargs: Keyword arguments passed to parent
        """
        self._module = module
        self.max_bacon = max_bacon
        self.max_module_depth = max_module_depth
        self.only_cycles = only_cycles
        self.clusters = clusters
        super().__init__(**kwargs)

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            module=self.module,
            max_bacon=self.max_bacon,
            max_module_depth=self.max_module_depth,
            only_cycles=self.only_cycles,
            clusters=self.clusters,
            _filter_empty=True,
            _filter_false=True,
        )

    @property
    def module(self):
        match self._module:
            case str() | os.PathLike:
                return self._module
            case types.ModuleType():
                return inspecthelpers.get_file(self._module)
            case _:
                return self.ctx.metadata.distribution_name

    def _to_markdown(self) -> str:
        content = get_dependency_svg(
            self.module,
            max_bacon=self.max_bacon,
            max_module_depth=self.max_module_depth,
            only_cycles=self.only_cycles,
            clusters=self.clusters,
        )
        content = insert_links(content, self.ctx.links.inv_manager)
        return f"<body>\n\n{content}\n\n</body>\n"

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkPyDeps("mknodes/data/tools.py", max_bacon=1, max_module_depth=1)
        page += mk.MkReprRawRendered(node)


if __name__ == "__main__":
    node = MkPyDeps.with_default_context("mknodes/data/tools.py", max_bacon=1)
    print(node)



from __future__ import annotations

from typing import Any

from mknodes.basenodes import mknode, mktabbed, mktabs
from mknodes.utils import log, reprhelpers


logger = log.get_logger(__name__)


class MkReprRawRendered(mktabbed.MkTabbed):
    """Node showing a tabbed block to visualize a node in different representations.

    It contains a tab for the repr, one for the rendered output,
    one for the markdown and a Repr tree in case the node has children.
    """

    ICON = "material/presentation"

    def __init__(
        self,
        node: mknode.MkNode,
        select_tab: int | str | None = 3,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            node: Node to show an example for
            select_tab: Tab which should be selected initially
            kwargs: Keyword arguments passed to parent
        """
        self.node = node
        super().__init__(tabs={}, select_tab=select_tab, **kwargs)

    def __repr__(self):
        return reprhelpers.get_repr(self, node=self.node)

    @property
    def items(self):
        import mknodes as mk

        html_node = self.node.__copy__()
        html_node.as_html = True
        tabs: dict[str, str | mk.MkNode] = dict(  # type: ignore[annotation-unchecked]
            Repr=mk.MkCode(repr(self.node)),
            Raw=mk.MkCode(self.node, language="markdown"),
            Html=mk.MkCode(html_node, language="html"),
            Rendered=self.node.__copy__(),
        )
        if len(self.node.children) > 0:
            tabs["Repr tree"] = mk.MkTreeView(self.node)
        items = [mktabs.MkTab(k, v, parent=self) for k, v in tabs.items()]
        items[0].new = True
        if self.select_tab is not None:
            pos = (
                self._get_tab_pos(self.select_tab)
                if isinstance(self.select_tab, str)
                else self.select_tab
            )
            items[pos].select = True
        return items

    @items.setter
    def items(self, value):
        pass

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        example_node = mk.MkAdmonition("Some text")
        node = MkReprRawRendered(node=example_node)
        page += node
        page += MkReprRawRendered(node)


if __name__ == "__main__":
    import mknodes as mk

    example_node = mk.MkAdmonition("Some text")
    node = MkReprRawRendered(node=example_node)
    print(node)



from __future__ import annotations

import mknodes as mk


EXPECTED = """<figure markdown>
  ![Image title](something.png)
  <figcaption>Caption</figcaption>
</figure>
"""


def test_image():
    image = mk.MkImage(path="something.png", caption="Caption", title="Image title")
    assert str(image) == EXPECTED



from __future__ import annotations

import pathlib

import pytest

# from responsemock import utils
import mknodes as mk

from mknodes.manual import root


RESPONSE_1 = """{
  "name": "mkdocstrings",
  "url": "https://api.github.com/repos/mkdocstrings/mkdocstrings",
  "size": 3249,
  "default_branch": "main"
}"""

RESPONSE_2 = """{
  "sha": "0a90a474c8dcbd95821700d7dab63f03e392c40f",
  "url": "https://api.github.com/repos/mkdocstrings/mkdocstrings/git/trees/0a90a474c8dcbd95821700d7dab63f03e392c40f",
  "tree": [
    {
      "path": "scripts",
      "mode": "040000",
      "type": "tree",
      "sha": "b30cc0fde9b9684fc1cdbe8238161c4d85202bcb",
      "url": "https://api.github.com/repos/mkdocstrings/mkdocstrings/git/trees/b30cc0fde9b9684fc1cdbe8238161c4d85202bcb"
    }
  ],
  "truncated": false
}"""

RESPONSE_3 = """{
  "sha": "b30cc0fde9b9684fc1cdbe8238161c4d85202bcb",
  "url": "https://api.github.com/repos/mkdocstrings/mkdocstrings/git/trees/b30cc0fde9b9684fc1cdbe8238161c4d85202bcb",
  "tree": [
    {
      "path": "gen_credits.py",
      "mode": "100644",
      "type": "blob",
      "sha": "bc01c0bd7d85b2820b0d16d1212930b7f5fc2ff0",
      "size": 4734,
      "url": "https://api.github.com/repos/mkdocstrings/mkdocstrings/git/blobs/bc01c0bd7d85b2820b0d16d1212930b7f5fc2ff0"
    }]}"""


@pytest.fixture(scope="session")
def test_data_dir():
    return pathlib.Path(__file__).parent / "data"


@pytest.fixture(scope="session")
def resources_dir():
    return pathlib.Path(__file__).parent.parent / "mknodes/resources/"


@pytest.fixture(scope="session")
def mknodes_project():
    theme = mk.MaterialTheme()
    proj = mk.Project(theme=theme)
    # with utils.response_mock(
    #     [
    #         (
    #             "GET https://raw.githubusercontent.com/phil65/mknodes/main/README.md ->"
    #             ' 200 :{"default_branch":"main"}'
    #         ),
    #         (
    #             "GET https://api.github.com/repos/mkdocstrings/mkdocstrings -> 200 :"
    #             f" {RESPONSE_1}"
    #         ),
    #         (
    #             "GET https://api.github.com/repos/mkdocstrings/mkdocstrings/git/trees/main"
    #             f" -> 200 :{RESPONSE_2}"
    #         ),
    #         (
    #             "GET https://api.github.com/repos/mkdocstrings/mkdocstrings/git/trees/b30cc0fde9b9684fc1cdbe8238161c4d85202bcb"
    #             f" -> 200 :{RESPONSE_3}"
    #         ),
    #     ],
    # ):
    root.build(proj)
    return proj



from __future__ import annotations

from typing import Any, Literal

from mknodes.basenodes import mkcontainer, mknode
from mknodes.data import datatypes
from mknodes.utils import inspecthelpers, log, reprhelpers, resources


logger = log.get_logger(__name__)


class MkCommentedCode(mkcontainer.MkContainer):
    """Node which displays a list of code / comment blocks for given code.

    Lines beginning with # are shown in dedicated blocks and can be used to
    inline-explain the code. Lines can be hidden by ending a line with "##".
    """

    ICON = "material/code-json"
    STATUS = "new"

    REQUIRED_EXTENSIONS = [
        resources.Extension("pymdownx.highlight"),
        resources.Extension("pymdownx.snippets"),
        resources.Extension("pymdownx.superfences"),
    ]

    def __init__(
        self,
        code: str | datatypes.HasCodeType,
        language: str = "py",
        *,
        linenums: int | None = None,
        style: Literal["text", "admonition", "bubble"] = "bubble",
        header: str = "",
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            code: Code to show
            language: language for syntax highlighting
            linenums: If set, use as start linenumber
            style: Comment style
            header: Section header
            kwargs: Keyword arguments passed to parent
        """
        self._code = code
        self.language = language
        self.title = ""
        self.linenums = linenums
        self._style = style
        super().__init__(content=None, header=header, **kwargs)

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            code=self._code,
            language=self.language,
            linenums=self.linenums,
            style=self._style,
            _filter_empty=True,
            _filter_false=True,
        )

    @property
    def code(self) -> str:
        match self._code:
            case str():
                return self._code
            case _:
                return inspecthelpers.get_source(self._code)

    @property
    def comment_class(self) -> type[mknode.MkNode]:
        import mknodes as mk

        match self._style:
            case "text":
                return mk.MkText
            case "bubble":
                return mk.MkSpeechBubble
            case "admonition":
                return mk.MkAdmonition
            case _:
                raise TypeError(self._style)

    @property
    def items(self):
        import mknodes as mk

        if not self.code:
            return {}
        section: list[str] = []
        sections: list[mk.MkNode] = []
        mode = ""
        line_num = self.linenums or 0
        for i, line in enumerate(self.code.split("\n"), start=line_num):
            if not line.strip() or line.rstrip().endswith("##"):
                continue
            if line.strip().startswith("#"):
                if mode == "code":
                    code = "\n".join(section)
                    start_line = line_num if self.linenums else None
                    node = mk.MkCode(code, linenums=start_line)
                    sections.append(node)
                    section = []
                    line_num = i
                section.append(line.strip().removeprefix("#")[1:])
                mode = "comment"
            elif not line.strip().startswith("#"):
                if mode == "comment":
                    text = "\n".join(section)
                    sections.append(self.comment_class(text))
                    section = []
                    line_num = i
                section.append(line)
                mode = "code"
        if mode == "code":
            code = "\n".join(section)
            start_line = line_num if self.linenums else None
            node = mk.MkCode(code, linenums=start_line)
            sections.append(node)
        elif mode == "comment":
            text = "\n".join(section)
            sections.append(self.comment_class(text))
        for sect in sections:
            sect.parent = self
        return sections

    @items.setter
    def items(self, value):
        pass

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        # Comment sections automatically get converted to non-codeblock sections.
        # That way you can explain your code in-line.

        # ## you can use headers.

        node = MkCommentedCode(MkCommentedCode.create_example_page)
        page += mk.MkReprRawRendered(node, header="### Regular")
        # !!! note
        #     Admonitions and everything else work, too.
        #

        node = MkCommentedCode(MkCommentedCode.create_example_page, style="admonition")
        page += mk.MkReprRawRendered(node, header="### Admonitions")
        node = MkCommentedCode(MkCommentedCode.create_example_page, style="text")
        page += mk.MkReprRawRendered(node, header="### Plain text")


if __name__ == "__main__":
    from mknodes.manual import root

    node = MkCommentedCode(root.build)
    print(node)



from __future__ import annotations

from collections.abc import Callable
import pathlib

from typing import cast

from mknodes.data import treestyles
from mknodes.treelib import node
from mknodes.utils import fsspecpath, log


logger = log.get_logger(__name__)


class FileTreeNode(node.Node):
    def __init__(self, path, **kwargs):
        self.path = path
        self.name = self.path.name
        super().__init__(**kwargs)

    def __repr__(self):
        return f"{self.name}/" if self.path.is_dir() else f"{self.name}"

    @classmethod
    def from_folder(
        cls,
        folder: pathlib.Path | fsspecpath.FsSpecPath,
        *,
        predicate: Callable | None = None,
        exclude_folders: list[str] | None = None,
        sort: bool = True,
        max_items: int | None = None,
        maximum_depth: int | None = None,
        parent: FileTreeNode | None = None,
    ):
        node = cls(folder, parent=parent)
        children = list(folder.iterdir())
        if sort:
            children = sorted(children, key=lambda s: str(s).lower())
        for path in children:
            if predicate and not predicate(path):
                continue
            if exclude_folders and path.name in exclude_folders and path.is_dir():
                continue
            if path.is_file():
                child = cls(path, parent=node)
            else:
                if maximum_depth is not None and maximum_depth < node.depth + 1:
                    continue
                child = FileTreeNode.from_folder(
                    path,
                    parent=node,
                    predicate=predicate,
                    max_items=max_items,
                    maximum_depth=maximum_depth,
                    exclude_folders=exclude_folders,
                )
            if max_items is not None:
                if max_items > 0:
                    max_items -= 1
                else:
                    break

            node.append_child(child)
        return node

    def get_folder_count(self) -> int:
        return sum(i.path.is_folder() for i in self.descendants)

    def get_file_count(self) -> int:
        return sum(i.path.is_file() for i in self.descendants)

    def get_tree_repr(
        self,
        max_depth: int | None = None,
        style: treestyles.TreeStyleStr | tuple[str, str, str, str] | None = None,
        show_icon: bool = True,
    ) -> str:
        lines = []
        for pre_str, fill_str, _node in self.yield_tree(
            max_depth=max_depth,
            style=style or "ascii",
        ):
            _node = cast(FileTreeNode, _node)
            if show_icon and _node.path.is_dir():
                icon = "📁"
            elif show_icon:
                icon = "📄"
            else:
                icon = ""
            lines.append(f"{pre_str}{fill_str}{icon}{_node!r}")
        return repr(self) + "\n" + "\n".join(lines[1:])


if __name__ == "__main__":
    folder = FileTreeNode.from_folder(
        pathlib.Path(),
        exclude_folders=["__pycache__", ".git", ".mypy_cache"],
        sort=False,
        maximum_depth=2,
    )
    logger.warning(folder.get_tree_repr())
    # for node in folder.descendants:
    #     logger.warning(node.displayable())
    # print(get_tree_repr(folder))



import os

import mknodes as mk


BASE_NODES = [
    mk.MkNode,
    mk.MkText,
    mk.MkHeader,
    mk.MkCompactAdmonition,
    mk.MkCritic,
    mk.MkLink,
    mk.MkKeys,
    mk.MkProgressBar,
    mk.MkSpeechBubble,
    mk.MkJinjaText,
    mk.MkJinjaTemplate,
]

IMAGE_NODES = [
    mk.MkImage,
    mk.MkImageCompare,
    mk.MkImageSlideshow,
    mk.MkBadge,
    mk.MkIcon,
    mk.MkMaterialBadge,
    mk.MkBinaryImage,
    mk.MkCard,
]

PRESENTATION_NODES = [
    mk.MkTreeView,
    mk.MkPrettyPrint,
    mk.MkReprRawRendered,
    mk.MkCodeImage,
    mk.MkDiagram,
    mk.MkTimeline,
]

DOCUMENTATION_NODES = [
    mk.MkClassDiagram,
    mk.MkDocStrings,
    mk.MkCommentedCode,
    mk.MkConfigSetting,
    mk.MkClassTable,
    mk.MkModuleTable,
    mk.MkPluginFlow,
    mk.MkArgParseHelp,
    mk.MkClickDoc,
]

BLOCK_NODES = [
    mk.MkBlock,
    mk.MkAdmonitionBlock,
    mk.MkDetailsBlock,
    mk.MkHtmlBlock,
    mk.MkTabbedBlocks,
]

ABOUT_THE_PROJECT_NODES = [
    mk.MkChangelog,
    mk.MkCodeOfConduct,
    mk.MkLicense,
    mk.MkDependencyTable,
    mk.MkInstallGuide,
    mk.MkCommitConventions,
    mk.MkPullRequestGuidelines,
    mk.MkDevEnvSetup,
    mk.MkDevTools,
    mk.MkShields,
    mk.MkMetadataBadges,
    mk.MkModuleOverview,
    mk.MkPipDepTree,
]

if os.environ.get("CI"):
    ABOUT_THE_PROJECT_NODES.append(mk.MkPyDeps)

CONTAINER_NODES = [
    mk.MkBlockQuote,
    mk.MkAdmonition,
    mk.MkContainer,
    mk.MkGrid,
    mk.MkCode,
    mk.MkList,
    mk.MkTable,
    mk.MkHtmlTable,
    mk.MkDefinitionList,
    # mk.MkTab,
    mk.MkTabbed,
    mk.MkAnnotations,
    mk.MkFootNotes,
    mk.MkShowcase,
    mk.MkTaskList,
]

SPECIAL_NODES = [
    mk.MkSnippet,
    mk.MkInclude,
    mk.MkIFrame,
    mk.MkCommandOutput,
    mk.MkCallable,
]


def create_section_for_nodes(
    nav: mk.MkNav,
    klasses: list[type[mk.MkNode]],
) -> mk.MkTable:
    table = mk.MkTable(columns=["Node", "Docstrings", "Markdown extensions"])
    for kls in klasses:
        if "create_example_page" in kls.__dict__:
            page = nav.add_page(kls.__name__, icon=kls.ICON)
            create_class_page(kls, page)
            link = mk.MkLink(page, kls.__name__, icon=kls.ICON)
            extensions = ", ".join(f"`{i}`" for i in kls.REQUIRED_EXTENSIONS)
            table.add_row((link, kls.__doc__, extensions))
    return table


def create_class_page(kls: type[mk.MkNode], page: mk.MkPage):
    page += mk.MkCode.for_object(kls.create_example_page, extract_body=True)
    page += "## Examples"
    if kls.STATUS:
        page.metadata.status = kls.STATUS
    elif kls.JS_FILES:
        page.metadata.status = "js"
    elif kls.CSS:
        page.metadata.status = "css"
    kls.create_example_page(page)
    page.created_by = create_class_page


nav = mk.MkNav("The nodes")


@nav.route.page(is_index=True)
def _(page: mk.MkPage):
    page += mk.MkJinjaTemplate("nodes_index.jinja")


@nav.route.nav("Base nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, BASE_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Image nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, IMAGE_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Container nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, CONTAINER_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Presentation nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, PRESENTATION_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Documentation nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, DOCUMENTATION_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("About-the-project nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, ABOUT_THE_PROJECT_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Special nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, SPECIAL_NODES)
    page.created_by = create_section_for_nodes


@nav.route.nav("Block nodes")
def _(nav: mk.MkNav):
    page = nav.add_page(is_index=True, hide="toc")
    page += create_section_for_nodes(nav, BLOCK_NODES)
    page.created_by = create_section_for_nodes



from __future__ import annotations

import dataclasses

from typing import Literal

from mknodes.utils import log


logger = log.get_logger(__name__)

ConventionTypeStr = Literal["conventional_commits", "basic"]

CommitTypeStr = Literal[
    "build",
    "chore",
    "ci",
    "deps",
    "doc",
    "docs",
    "feat",
    "fix",
    "ref",
    "revert",
    "add",
    "change",
    "remove",
    "merge",
    "perf",
    "refactor",
    "style",
    "test",
    "tests",
]

TYPE_DESCRIPTIONS: dict[CommitTypeStr, str] = {
    "build": "About packaging, building wheels, etc.",
    "chore": "About packaging or repo/files management.",
    "ci": "About Continuous Integration.",
    "deps": "Dependencies update.",
    "doc": "About documentation.",
    "docs": "About documentation.",
    "feat": "New feature.",
    "fix": "Bug fix.",
    "ref": "Code refactoring.",
    "revert": "Code revert.",
    "add": "Code Addition.",
    "change": "Code change.",
    "remove": "Code removal.",
    "merge": "Code merge.",
    "perf": "About performance.",
    "refactor": "Changes that are not features or bug fixes.",
    "style": "A change in code style/format.",
    "test": "About tests.",
    "tests": "About tests.",
}


@dataclasses.dataclass
class CommitConvention:
    name: str
    display_name: str
    types: set[CommitTypeStr]
    badge: str = ""
    website: str | None = None


basic = CommitConvention(
    name="basic",
    display_name="Basic Style",
    types={"add", "fix", "change", "remove", "merge", "doc"},
)

angular = CommitConvention(
    name="angular",
    display_name="Angular Style",
    website="https://gist.github.com/stephenparish/9941e89d80e2bc58a153",
    types={
        "build",
        "chore",
        "ci",
        "deps",
        "doc",
        "docs",
        "feat",
        "fix",
        "perf",
        "ref",
        "refactor",
        "revert",
        "style",
        "test",
        "tests",
    },
)

conventional_commits = CommitConvention(
    name="conventional_commmits",
    display_name="Conventional commits",
    website="https://www.conventionalcommits.org/en/v1.0.0/",
    badge="https://img.shields.io/badge/Conventional%20Commits-1.0.0-yellow.svg",
    types={
        "build",
        "chore",
        "ci",
        "deps",
        "doc",
        "docs",
        "feat",
        "fix",
        "perf",
        "ref",
        "refactor",
        "revert",
        "style",
        "test",
        "tests",
    },
)



from __future__ import annotations

import ast
import datetime
import functools
import importlib

from importlib import util
import json
import os
import time
import tomllib
from typing import Any

import jinja2

from markupsafe import Markup
import tomli_w

from mknodes import paths
from mknodes.utils import helpers, icons, inspecthelpers, log, pathhelpers, yamlhelpers


logger = log.get_logger(__name__)


@jinja2.pass_context
def url_filter(context, value: str) -> str:
    """A Template filter to normalize URLs."""
    return pathhelpers.normalize_url(
        str(value),
        url=page.url if (page := context.get("page")) else None,
        base=context["base_url"],
    )


@jinja2.pass_context
def script_tag_filter(context, extra_script):
    """Converts an ExtraScript value to an HTML <script> tag line."""
    html = '<script src="{0}"'
    if not isinstance(extra_script, str):
        if extra_script.type:
            html += ' type="{1.type}"'
        if extra_script.defer:
            html += " defer"
        if extra_script.async_:
            html += " async"
    html += "></script>"
    return Markup(html).format(url_filter(context, str(extra_script)), extra_script)


def evaluate(
    text: str,
    context: dict[str, Any] | None = None,
    return_val: str | None = None,
):
    """Evaluate several lines of input, returning the result of the last line."""
    now = time.time()
    if context is None:
        context = locals()
    code = str(text) if return_val is None else f"{text}\n{return_val}"
    logger.debug("Evaluating code:\n%s", text)
    tree = ast.parse(code)
    eval_expr = ast.Expression(tree.body[-1].value)  # type: ignore
    # exec_expr = ast.Module(tree.body[:-1])  # type: ignore
    exec_expr = ast.parse("")
    exec_expr.body = tree.body[:-1]
    compiled = compile(exec_expr, "file", "exec")
    exec(compiled, context)
    val = eval(compile(eval_expr, "file", "eval"), context)
    logger.debug("Code evaluation took %s seconds.", time.time() - now)
    return val


ENV_GLOBALS = {
    "log": log.log_stream.getvalue,
    "now": datetime.datetime.now,
    "str": str,
    "importlib": importlib,
    "inspecthelpers": inspecthelpers,
    "resources_dir": paths.RESOURCES,
}
ENV_FILTERS = {
    "get_icon_svg": icons.get_icon_svg,
    "styled": helpers.styled,
    "str": str,
    "rstrip": str.rstrip,
    "lstrip": str.lstrip,
    "removesuffix": str.removesuffix,
    "removeprefix": str.removeprefix,
    "issubclass": issubclass,
    "isinstance": isinstance,
    "hasattr": hasattr,
    "evaluate": evaluate,
    "partial": functools.partial,
    "dump_yaml": yamlhelpers.dump_yaml,
    "dump_json": json.dumps,
    "dump_toml": tomli_w.dumps,
    "load_json": json.loads,
    "joad_toml": tomllib.loads,
    "load_yaml": yamlhelpers.load_yaml,
    "load_file": pathhelpers.load_file_cached,
    "path_join": os.path.join,
    "url": url_filter,
    "script_tag": script_tag_filter,
}


def get_globals():
    import mknodes as mk

    node_klasses = {i: getattr(mk, i) for i in mk.__all__}
    return dict(mk=node_klasses, _mk=node_klasses) | ENV_GLOBALS


def get_filters():
    import mknodes as mk

    node_klasses = {i: getattr(mk, i) for i in mk.__all__}
    return ENV_FILTERS | node_klasses


def set_markdown_exec_namespace(variables: dict[str, Any], namespace: str = "mknodes"):
    if util.find_spec("markdown_exec"):
        from markdown_exec.formatters import python

        python._sessions_globals[namespace] = variables


if __name__ == "__main__":
    a = evaluate("import mknodes\nmknodes.MkHeader('Hello')")
    print(a)



from __future__ import annotations

from collections.abc import Iterable, Mapping
import dataclasses
import os


MD_ESCAPE_CHARS = tuple("!#()*+-[\\]_`{}")


@dataclasses.dataclass
class Item:
    level: int
    title: str
    filename: str | None


class NavBuilder:
    """An object representing MkDocs navigation.

    Consists of files under nested
    sequences of titles, which are treated like paths.
    """

    def __init__(self):
        self._data = {}

    def __setitem__(self, keys: str | tuple[str, ...], value: str):
        """Add a link to a file into the nav, under the sequence of titles.

        For example, writing `nav["Foo", "Bar"] = "foo/bar.md"` would mean creating a nav:
        `{"Foo": {"Bar": "foo/bar.md"}}`.

        Then, writing `nav["Foo", "Another"] = "test.md"` would merge with the existing
        sections where possible:
        `{"Foo": {"Bar": "foo/bar.md", "Another": "test.md"}}`.
        """
        if isinstance(keys, str):
            keys = (keys,)
        cur = self._data
        if not keys:
            msg = f"The navigation path must not be empty (got {keys!r})"
            raise ValueError(msg)
        for key in keys:
            if not isinstance(key, str):
                msg = f"The navigation path must consist of strings, got {type(key)}"
                raise TypeError(msg)
            if not key:
                msg = f"The navigation name parts must not be empty (got {keys!r})"
                raise ValueError(msg)
            cur = cur.setdefault(key, {})
        cur[None] = os.fspath(value)

    def items(self) -> Iterable[Item]:
        return self._items(self._data, 0)

    @classmethod
    def _items(cls, data: Mapping, level: int) -> Iterable[Item]:
        for key, value in data.items():
            if key is not None:
                yield Item(level=level, title=key, filename=value.get(None))
                yield from cls._items(value, level + 1)

    def build_literate_nav(self, indentation: int | str = "") -> Iterable[str]:
        """Convert data to a literate-nav formatted markdown list.

        Arguments:
            indentation: Initial indentation
        """
        if isinstance(indentation, int):
            indentation = " " * indentation
        for item in self.items():
            line = item.title
            if line.startswith(MD_ESCAPE_CHARS):
                line = "\\" + line
            if item.filename is not None:
                line = f"[{line}]({item.filename})"
            yield indentation + "    " * item.level + "* " + line + "\n"



from __future__ import annotations

import mknodes as mk


def test_modes():
    mk.MkClassDiagram(mk.MkClassDiagram, mode="mro")
    mk.MkClassDiagram(mk.MkClassDiagram, mode="subclasses")
    mk.MkClassDiagram(mk.MkClassDiagram, mode="baseclasses")



from __future__ import annotations

import functools
import os
import pathlib

from mknodes.utils import log


logger = log.get_logger(__name__)


def _walk_directory(directory: os.PathLike | str, tree) -> None:
    """Recursively build a Tree with directory contents."""
    # Sort dirs first then by filename
    from rich.filesize import decimal
    from rich.markup import escape
    from rich.text import Text

    paths = sorted(
        pathlib.Path(directory).iterdir(),
        key=lambda path: (path.is_file(), path.name.lower()),
    )
    for path in paths:
        # Remove hidden files
        if path.name.startswith("."):
            continue
        if path.is_dir():
            style = "dim" if path.name.startswith("__") else ""
            branch = tree.add(
                "[bold magenta]:open_file_folder: [link"
                f" file://{path}]{escape(path.name)}",
                style=style,
                guide_style=style,
            )
            _walk_directory(path, branch)
        else:
            text_filename = Text(path.name, "green")
            text_filename.highlight_regex(r"\..*$", "bold red")
            text_filename.stylize(f"link file://{path}")
            file_size = path.stat().st_size
            text_filename.append(f" ({decimal(file_size)})", "blue")
            icon = "🐍 " if path.suffix == ".py" else "📄 "
            tree.add(Text(icon) + text_filename)


@functools.cache
def get_folder_tree_svg(
    directory: str | os.PathLike,
    width: int = 80,
    title: str = "",
) -> str:
    from rich.console import Console
    from rich.tree import Tree

    tree = Tree(
        f":open_file_folder: [link file://{directory}]{directory}",
        guide_style="bold bright_blue",
    )
    _walk_directory(directory, tree)
    console = Console(record=True, width=width, markup=True)
    with console.capture() as _capture:
        # renderable = Padding(tree, (0,), expand=False)
        console.print(tree, markup=True)
    text = console.export_svg(title=title)
    return f"<body>\n\n{text}\n\n</body>\n\n"


@functools.cache
def get_svg_for_code(
    text: str,
    title: str = "",
    width: int = 80,
    language: str = "python",
    pygments_style: str = "material",
) -> str:
    from rich.console import Console
    from rich.padding import Padding
    from rich.syntax import Syntax

    # with console.capture() as _capture:
    with open(os.devnull, "w") as devnull:  # noqa: PTH123
        console = Console(record=True, width=width, file=devnull, markup=False)
        renderable = Syntax(text, lexer=language, theme=pygments_style)
        renderable = Padding(renderable, (0,), expand=False)
        console.print(renderable, markup=False)
    return console.export_svg(title=title)


if __name__ == "__main__":
    import rich

    strings = get_folder_tree_svg(".")
    rich.inspect(strings)



from __future__ import annotations

import pathlib

from typing import TYPE_CHECKING, Any, Literal

from mknodes.basenodes import mknode
from mknodes.utils import helpers, log, reprhelpers


if TYPE_CHECKING:
    import mknodes as mk

logger = log.get_logger(__name__)


class MkImage(mknode.MkNode):
    """Image including optional caption."""

    ICON = "material/image"
    ATTR_LIST_SEPARATOR = ""

    def __init__(
        self,
        path: str,
        *,
        link: str | mk.MkPage | mk.MkNav | None = None,
        caption: str = "",
        title: str = "",
        align: Literal["left", "right"] | None = None,
        width: int | None = None,
        lazy: bool = False,
        path_dark_mode: str | None = None,
        **kwargs: Any,
    ):
        """Constructor.

        Arguments:
            path: path of the image
            link: Optional url the image should link to
            caption: Image caption
            title: Image title
            align: Image alignment
            width: Image width in pixels
            lazy: Whether to lazy-load image
            path_dark_mode: Optional alternative image for dark mode
            kwargs: Keyword arguments passed to parent
        """
        super().__init__(**kwargs)
        self.title = title
        self.caption = caption
        self.target = link
        self.align = align
        self.width = width
        self.lazy = lazy
        self._path_dark_mode = path_dark_mode
        self._path = path

    @property
    def path(self) -> str:
        if helpers.is_url(self._path):
            return self._path
        # TODO: linkreplacer doesnt work yet with full path
        return pathlib.Path(self._path).name  # this should not be needed.

    @property
    def path_dark_mode(self):
        match self._path_dark_mode:
            case str() if helpers.is_url(self._path_dark_mode):
                return self._path_dark_mode
            case str():
                return pathlib.Path(self._path_dark_mode).name
            case _:
                return None

    def __repr__(self):
        return reprhelpers.get_repr(
            self,
            path=self.path,
            caption=self.caption,
            link=self.url,
            align=self.align,
            width=self.width,
            lazy=self.lazy,
            path_dark_mode=self.path_dark_mode,
            _filter_empty=True,
            _filter_false=True,
        )

    @property
    def url(self) -> str:
        return self.ctx.links.get_url(self.target) if self.target else ""

    def _to_markdown(self) -> str:
        if not self.path_dark_mode:
            markdown_link = self._build(self.path)
        else:
            link_2 = self._build(self.path, "light")
            link_1 = self._build(self.path_dark_mode, "dark")
            markdown_link = f"{link_1} {link_2}"
        if not self.caption:
            return markdown_link
        lines = [
            "<figure markdown>",
            f"  {markdown_link}",
            f"  <figcaption>{self.caption}</figcaption>",
            "</figure>",
        ]
        return "\n".join(lines) + "\n"

    def _build(self, path, mode: Literal["light", "dark"] | None = None) -> str:
        if mode:
            path += f"#only-{mode}"
        markdown_link = f"![{self.title}]({path})"
        if self.align:
            markdown_link += f"{{ align={self.align} }}"
        if self.width:
            markdown_link += f'{{ width="{self.width}" }}'
        if self.lazy:
            markdown_link += "{ loading=lazy }"
        if self.target:
            markdown_link = f"[{markdown_link}]({self.url})"
        return markdown_link

    @classmethod
    def create_example_page(cls, page):
        import mknodes as mk

        node = MkImage(path="https://picsum.photos/200", caption="Dummy image")
        page += mk.MkReprRawRendered(node, header="### With caption")

        node = MkImage(path="https://picsum.photos/200", align="right")
        page += mk.MkReprRawRendered(node, header="### Aligned")

        node = MkImage(path="https://picsum.photos/200", width=500)
        page += mk.MkReprRawRendered(node, header="### Fixed width")

        node = MkImage(path="https://picsum.photos/200", link="https://www.google.com")
        page += mk.MkReprRawRendered(node, header="### Linked")

        node = MkImage(
            path="https://picsum.photos/200",
            link="https://www.google.com",
            path_dark_mode="https://picsum.photos/300",
        )
        page += mk.MkReprRawRendered(node, header="### Separate dark mode")


if __name__ == "__main__":
    img = MkImage("Some path", link="http://www.google.de", title="test")
    print(img)
